import { ProjectParser } from '#lib/structures/ProjectParser';
import { ClassParser } from '#lib/structures/class-parser';
export var MigrationStatus;
(function (MigrationStatus) {
    MigrationStatus[MigrationStatus["Success"] = 0] = "Success";
    MigrationStatus[MigrationStatus["Failed"] = 1] = "Failed";
    MigrationStatus[MigrationStatus["Latest"] = 2] = "Latest";
})(MigrationStatus || (MigrationStatus = {}));
function isSemverPatchDiff(version1, version2) {
    return version1.split('.').slice(0, 2).join('.') === version2.split('.').slice(0, 2).join('.');
}
export function migrateToLatest(projectJson, path) {
    let currentVersion = `v${'typeDocJsonParserVersion' in projectJson ? projectJson.typeDocJsonParserVersion : '2.0.0'}`;
    const latestVersion = `v${ProjectParser.version}`;
    let result;
    while (currentVersion !== latestVersion) {
        if (isSemverPatchDiff(currentVersion, latestVersion)) {
            projectJson = latest(projectJson);
            result = { status: MigrationStatus.Latest, path, name: projectJson.name, version: projectJson.version };
            currentVersion = latestVersion;
            break;
        }
        const migration = migrations.find((migration) => migration.from.includes(currentVersion));
        if (!migration) {
            result = {
                status: MigrationStatus.Failed,
                path,
                from: currentVersion,
                name: projectJson.name,
                version: projectJson.version
            };
            break;
        }
        projectJson = migration.run(projectJson);
        currentVersion = migration.to;
    }
    return result ?? { status: MigrationStatus.Success, path, data: projectJson };
}
const migrations = [
    {
        from: ['v2.0.0', 'v2.0.1', 'v2.0.2'],
        to: 'v2.1.0',
        run(projectJson) {
            return {
                ...projectJson,
                typeDocJsonParserVersion: '2.1.0'
            };
        }
    },
    {
        from: ['v2.1.0'],
        to: 'v2.2.1',
        run(projectJson) {
            return {
                ...projectJson,
                typeDocJsonParserVersion: '2.2.1',
                version: null
            };
        }
    },
    {
        from: ['v2.2.0', 'v2.2.1'],
        to: 'v2.3.2',
        run(projectJson) {
            function migrateSignatureJson(signatureJson) {
                return {
                    ...signatureJson,
                    comment: {
                        description: null,
                        blockTags: [],
                        modifierTags: []
                    }
                };
            }
            return {
                ...projectJson,
                typeDocJsonParserVersion: '2.3.2',
                classes: projectJson.classes.map((classJson) => ({
                    ...classJson,
                    methods: classJson.methods.map((methodJson) => ({
                        ...methodJson,
                        signatures: methodJson.signatures.map(migrateSignatureJson)
                    }))
                })),
                functions: projectJson.functions.map((functionJson) => ({
                    ...functionJson,
                    signatures: functionJson.signatures.map(migrateSignatureJson)
                }))
            };
        }
    },
    {
        from: ['v2.3.0', 'v2.3.1', 'v2.3.2'],
        to: 'v3.0.0',
        run(projectJson) {
            function migrateSourceJson(sourceJson) {
                return { ...sourceJson, url: null };
            }
            function migrateClassJson(classJson) {
                return {
                    ...classJson,
                    source: migrateSourceJson(classJson.source),
                    construct: {
                        ...classJson.construct,
                        source: migrateSourceJson(classJson.construct.source)
                    },
                    properties: classJson.properties.map((propertyJson) => ({
                        ...propertyJson,
                        source: migrateSourceJson(propertyJson.source)
                    })),
                    methods: classJson.methods.map((methodJson) => ({
                        ...methodJson,
                        source: migrateSourceJson(methodJson.source)
                    }))
                };
            }
            function migrateConstantJson(constantJson) {
                return { ...constantJson, source: migrateSourceJson(constantJson.source) };
            }
            function migrateEnumJson(enumJson) {
                return {
                    ...enumJson,
                    source: migrateSourceJson(enumJson.source),
                    properties: enumJson.properties.map((propertyJson) => ({
                        ...propertyJson,
                        source: migrateSourceJson(propertyJson.source)
                    }))
                };
            }
            function migrateFunctionJson(functionJson) {
                return { ...functionJson, source: migrateSourceJson(functionJson.source) };
            }
            function migrateInterfaceJson(interfaceJson) {
                return {
                    ...interfaceJson,
                    source: migrateSourceJson(interfaceJson.source),
                    properties: interfaceJson.properties.map((propertyJson) => ({
                        ...propertyJson,
                        source: migrateSourceJson(propertyJson.source)
                    }))
                };
            }
            function migrateNamespaceJson(namespaceJson) {
                return {
                    ...namespaceJson,
                    source: migrateSourceJson(namespaceJson.source),
                    classes: namespaceJson.classes.map(migrateClassJson),
                    constants: namespaceJson.constants.map(migrateConstantJson),
                    enums: namespaceJson.enums.map(migrateEnumJson),
                    functions: namespaceJson.functions.map(migrateFunctionJson),
                    interfaces: namespaceJson.interfaces.map(migrateInterfaceJson),
                    namespaces: namespaceJson.namespaces.map(migrateNamespaceJson),
                    typeAliases: namespaceJson.typeAliases.map(migrateTypeAlias)
                };
            }
            function migrateTypeAlias(typeAliasJson) {
                return { ...typeAliasJson, source: migrateSourceJson(typeAliasJson.source) };
            }
            return {
                ...projectJson,
                typeDocJsonParserVersion: '3.0.0',
                readme: null,
                classes: projectJson.classes.map(migrateClassJson),
                constants: projectJson.constants.map(migrateConstantJson),
                enums: projectJson.enums.map(migrateEnumJson),
                functions: projectJson.functions.map(migrateFunctionJson),
                interfaces: projectJson.interfaces.map(migrateInterfaceJson),
                namespaces: projectJson.namespaces.map(migrateNamespaceJson),
                typeAliases: projectJson.typeAliases.map(migrateTypeAlias)
            };
        }
    },
    {
        from: ['v3.0.0'],
        to: 'v3.1.0',
        run(projectJson) {
            function migrateInterfaceJson(interfaceJson) {
                return { ...interfaceJson, methods: [] };
            }
            function migrateNamespaceJson(namespaceJson) {
                return {
                    ...namespaceJson,
                    interfaces: namespaceJson.interfaces.map(migrateInterfaceJson),
                    namespaces: namespaceJson.namespaces.map(migrateNamespaceJson)
                };
            }
            return {
                ...projectJson,
                interfaces: projectJson.interfaces.map(migrateInterfaceJson),
                namespaces: projectJson.namespaces.map(migrateNamespaceJson)
            };
        }
    },
    {
        from: ['v3.1.0'],
        to: 'v3.2.0',
        run(projectJson) {
            return {
                ...projectJson,
                typeDocJsonParserVersion: '3.2.0',
                changelog: null
            };
        }
    },
    {
        from: ['v3.2.0'],
        to: 'v4.0.0',
        run(projectJson) {
            function migrateClassJson(classJson) {
                return {
                    ...classJson,
                    construct: {
                        ...classJson.construct,
                        parentId: classJson.id
                    },
                    properties: classJson.properties.map((propertyJson) => ({
                        ...propertyJson,
                        parentId: classJson.id
                    })),
                    methods: classJson.methods.map((methodJson) => ({
                        ...methodJson,
                        parentId: classJson.id
                    }))
                };
            }
            function migrateEnumJson(enumJson) {
                return {
                    ...enumJson,
                    properties: enumJson.properties.map((propertyJson) => ({
                        ...propertyJson,
                        parentId: enumJson.id
                    }))
                };
            }
            function migrateInterfaceJson(interfaceJson) {
                return {
                    ...interfaceJson,
                    properties: interfaceJson.properties.map((propertyJson) => ({
                        ...propertyJson,
                        parentId: interfaceJson.id
                    })),
                    methods: interfaceJson.methods.map((methodJson) => ({
                        ...methodJson,
                        parentId: interfaceJson.id
                    }))
                };
            }
            function migrateNamespaceJson(namespaceJson) {
                return {
                    ...namespaceJson,
                    classes: namespaceJson.classes.map(migrateClassJson),
                    enums: namespaceJson.enums.map(migrateEnumJson),
                    interfaces: namespaceJson.interfaces.map(migrateInterfaceJson),
                    namespaces: namespaceJson.namespaces.map(migrateNamespaceJson)
                };
            }
            return {
                ...projectJson,
                typeDocJsonParserVersion: '4.0.0',
                classes: projectJson.classes.map(migrateClassJson),
                enums: projectJson.enums.map(migrateEnumJson),
                interfaces: projectJson.interfaces.map(migrateInterfaceJson),
                namespaces: projectJson.namespaces.map(migrateNamespaceJson)
            };
        }
    },
    {
        from: ['v4.0.0', 'v5.0.0', 'v5.0.1', 'v5.1.0', 'v5.2.0'],
        to: 'v6.0.0',
        run(projectJson) {
            function migrateParameterJson(parameterJson) {
                return {
                    ...parameterJson,
                    comment: {
                        description: null,
                        blockTags: [],
                        modifierTags: []
                    }
                };
            }
            function migrateClassJson(classJson) {
                return {
                    ...classJson,
                    typeParameters: [],
                    construct: {
                        ...classJson.construct,
                        parameters: classJson.construct.parameters.map(migrateParameterJson)
                    },
                    methods: classJson.methods.map((methodJson) => {
                        const { id, name, source, parentId, accessibility, abstract, signatures } = methodJson;
                        return {
                            id,
                            name,
                            source,
                            parentId,
                            accessibility,
                            abstract,
                            static: methodJson.static,
                            signatures: signatures.map((signatureJson) => ({
                                ...signatureJson,
                                parameters: signatureJson.parameters.map(migrateParameterJson)
                            }))
                        };
                    })
                };
            }
            function migrateEnumJson(enumJson) {
                const { id, name, comment, source, external, properties } = enumJson;
                return {
                    id,
                    name,
                    comment,
                    source,
                    external,
                    members: properties
                };
            }
            function migrateFunctionJson(functionJson) {
                return {
                    ...functionJson,
                    signatures: functionJson.signatures.map((signatureJson) => ({
                        ...signatureJson,
                        parameters: signatureJson.parameters.map(migrateParameterJson)
                    }))
                };
            }
            function migrateInterfaceJson(interfaceJson) {
                return {
                    ...interfaceJson,
                    methods: interfaceJson.methods.map((methodJson) => {
                        const { id, name, source, parentId, signatures } = methodJson;
                        return {
                            id,
                            name,
                            source,
                            parentId,
                            signatures: signatures.map((signatureJson) => ({
                                ...signatureJson,
                                parameters: signatureJson.parameters.map(migrateParameterJson)
                            }))
                        };
                    })
                };
            }
            function migrateNamespaceJson(namespaceJson) {
                return {
                    ...namespaceJson,
                    classes: namespaceJson.classes.map(migrateClassJson),
                    enums: namespaceJson.enums.map(migrateEnumJson),
                    functions: namespaceJson.functions.map(migrateFunctionJson),
                    interfaces: namespaceJson.interfaces.map(migrateInterfaceJson),
                    namespaces: namespaceJson.namespaces.map(migrateNamespaceJson),
                    typeAliases: namespaceJson.typeAliases,
                    variables: namespaceJson.constants
                };
            }
            return {
                ...projectJson,
                typeDocJsonParserVersion: '6.0.0',
                classes: projectJson.classes.map(migrateClassJson),
                enums: projectJson.enums.map(migrateEnumJson),
                functions: projectJson.functions.map(migrateFunctionJson),
                interfaces: projectJson.interfaces.map(migrateInterfaceJson),
                namespaces: projectJson.namespaces.map(migrateNamespaceJson),
                typeAliases: projectJson.typeAliases,
                variables: projectJson.constants
            };
        }
    },
    {
        from: ['v6.0.0', 'v6.0.1', 'v6.0.2'],
        to: 'v7.0.0',
        run(projectJson) {
            function migrateTypeParameter(typeParameter) {
                return {
                    ...typeParameter,
                    constraint: typeParameter.type
                };
            }
            function migrateClassJson(classJson) {
                return {
                    ...classJson,
                    typeParameters: classJson.typeParameters.map(migrateTypeParameter),
                    construct: {
                        ...classJson.construct,
                        accessibility: ClassParser.Accessibility.Public
                    },
                    methods: classJson.methods.map((methodJson) => ({
                        ...methodJson,
                        signatures: methodJson.signatures.map((signatureJson) => ({
                            ...signatureJson,
                            typeParameters: signatureJson.typeParameters.map(migrateTypeParameter)
                        }))
                    })),
                    properties: classJson.properties.map((propertyJson) => ({
                        ...propertyJson,
                        type: propertyJson.type
                    }))
                };
            }
            function migrateFunctionJson(functionJson) {
                return {
                    ...functionJson,
                    signatures: functionJson.signatures.map((signatureJson) => ({
                        ...signatureJson,
                        typeParameters: signatureJson.typeParameters.map(migrateTypeParameter)
                    }))
                };
            }
            function migrateInterfaceJson(interfaceJson) {
                return {
                    ...interfaceJson,
                    typeParameters: [],
                    methods: interfaceJson.methods.map((methodJson) => ({
                        ...methodJson,
                        signatures: methodJson.signatures.map((signatureJson) => ({
                            ...signatureJson,
                            typeParameters: signatureJson.typeParameters.map(migrateTypeParameter)
                        }))
                    }))
                };
            }
            function migrateNamespaceJson(namespaceJson) {
                return {
                    ...namespaceJson,
                    classes: namespaceJson.classes.map(migrateClassJson),
                    interfaces: namespaceJson.interfaces.map(migrateInterfaceJson),
                    functions: namespaceJson.functions.map(migrateFunctionJson),
                    namespaces: namespaceJson.namespaces.map(migrateNamespaceJson),
                    typeAliases: namespaceJson.typeAliases.map(migrateTypeAliasJson)
                };
            }
            function migrateTypeAliasJson(typeAliasJson) {
                return {
                    ...typeAliasJson,
                    typeParameters: typeAliasJson.typeParameters.map(migrateTypeParameter)
                };
            }
            return {
                ...projectJson,
                typeDocJsonParserVersion: '7.0.0',
                classes: projectJson.classes.map(migrateClassJson),
                functions: projectJson.functions.map(migrateFunctionJson),
                interfaces: projectJson.interfaces.map(migrateInterfaceJson),
                namespaces: projectJson.namespaces.map(migrateNamespaceJson),
                typeAliases: projectJson.typeAliases.map(migrateTypeAliasJson)
            };
        }
    },
    {
        from: ['v7.0.0', 'v7.0.1', 'v7.0.2'],
        to: 'v7.1.0',
        run(projectJson) {
            function migrateParameterJson(parameterJson) {
                return { ...parameterJson, optional: false };
            }
            function migrateClassJson(classJson) {
                return {
                    ...classJson,
                    construct: {
                        ...classJson.construct,
                        parameters: classJson.construct.parameters.map(migrateParameterJson)
                    },
                    methods: classJson.methods.map((methodJson) => ({
                        ...methodJson,
                        signatures: methodJson.signatures.map((signatureJson) => ({
                            ...signatureJson,
                            parameters: signatureJson.parameters.map(migrateParameterJson)
                        }))
                    }))
                };
            }
            function migrateFunctionJson(functionJson) {
                return {
                    ...functionJson,
                    signatures: functionJson.signatures.map((signatureJson) => ({
                        ...signatureJson,
                        parameters: signatureJson.parameters.map(migrateParameterJson)
                    }))
                };
            }
            function migrateInterfaceJson(interfaceJson) {
                return {
                    ...interfaceJson,
                    methods: interfaceJson.methods.map((methodJson) => ({
                        ...methodJson,
                        signatures: methodJson.signatures.map((signatureJson) => ({
                            ...signatureJson,
                            parameters: signatureJson.parameters.map(migrateParameterJson)
                        }))
                    }))
                };
            }
            function migrateNamespaceJson(namespaceJson) {
                return {
                    ...namespaceJson,
                    classes: namespaceJson.classes.map(migrateClassJson),
                    functions: namespaceJson.functions.map(migrateFunctionJson),
                    interfaces: namespaceJson.interfaces.map(migrateInterfaceJson),
                    namespaces: namespaceJson.namespaces.map(migrateNamespaceJson)
                };
            }
            return {
                ...projectJson,
                typeDocJsonParserVersion: '7.1.0',
                classes: projectJson.classes.map(migrateClassJson),
                functions: projectJson.functions.map(migrateFunctionJson),
                interfaces: projectJson.interfaces.map(migrateInterfaceJson),
                namespaces: projectJson.namespaces.map(migrateNamespaceJson)
            };
        }
    },
    {
        from: ['v7.1.0'],
        to: 'v7.2.0',
        run(projectJson) {
            function migrateParameterJson(parameterJson) {
                return { ...parameterJson, rest: false };
            }
            function migrateClassJson(classJson) {
                return {
                    ...classJson,
                    construct: {
                        ...classJson.construct,
                        parameters: classJson.construct.parameters.map(migrateParameterJson)
                    },
                    methods: classJson.methods.map((methodJson) => ({
                        ...methodJson,
                        signatures: methodJson.signatures.map((signatureJson) => ({
                            ...signatureJson,
                            parameters: signatureJson.parameters.map(migrateParameterJson)
                        }))
                    }))
                };
            }
            function migrateFunctionJson(functionJson) {
                return {
                    ...functionJson,
                    signatures: functionJson.signatures.map((signatureJson) => ({
                        ...signatureJson,
                        parameters: signatureJson.parameters.map(migrateParameterJson)
                    }))
                };
            }
            function migrateInterfaceJson(interfaceJson) {
                return {
                    ...interfaceJson,
                    methods: interfaceJson.methods.map((methodJson) => ({
                        ...methodJson,
                        signatures: methodJson.signatures.map((signatureJson) => ({
                            ...signatureJson,
                            parameters: signatureJson.parameters.map(migrateParameterJson)
                        }))
                    }))
                };
            }
            function migrateNamespaceJson(namespaceJson) {
                return {
                    ...namespaceJson,
                    classes: namespaceJson.classes.map(migrateClassJson),
                    functions: namespaceJson.functions.map(migrateFunctionJson),
                    interfaces: namespaceJson.interfaces.map(migrateInterfaceJson),
                    namespaces: namespaceJson.namespaces.map(migrateNamespaceJson)
                };
            }
            return {
                ...projectJson,
                typeDocJsonParserVersion: '7.2.0',
                classes: projectJson.classes.map(migrateClassJson),
                functions: projectJson.functions.map(migrateFunctionJson),
                interfaces: projectJson.interfaces.map(migrateInterfaceJson),
                namespaces: projectJson.namespaces.map(migrateNamespaceJson)
            };
        }
    },
    {
        from: ['v7.2.0'],
        to: 'v7.3.0',
        run(projectJson) {
            function migrateClassJson(classJson, namespaceParentId) {
                return { ...classJson, namespaceParentId };
            }
            function migrateEnumJson(enumJson, namespaceParentId) {
                return { ...enumJson, namespaceParentId };
            }
            function migrateFunctionJson(functionJson, namespaceParentId) {
                return { ...functionJson, namespaceParentId };
            }
            function migrateInterfaceJson(interfaceJson, namespaceParentId) {
                return { ...interfaceJson, namespaceParentId };
            }
            function migrateNamespaceJson(namespaceJson, namespaceParentId) {
                return {
                    ...namespaceJson,
                    classes: namespaceJson.classes.map((classJson) => migrateClassJson(classJson, namespaceJson.id)),
                    enums: namespaceJson.enums.map((enumJson) => migrateEnumJson(enumJson, namespaceJson.id)),
                    functions: namespaceJson.functions.map((functionJson) => migrateFunctionJson(functionJson, namespaceJson.id)),
                    interfaces: namespaceJson.interfaces.map((interfaceJson) => migrateInterfaceJson(interfaceJson, namespaceJson.id)),
                    namespaces: namespaceJson.namespaces.map((childNamespaceJson) => migrateNamespaceJson(childNamespaceJson, namespaceJson.id)),
                    typeAliases: namespaceJson.typeAliases.map((typeAliasJson) => migrateTypeAliasJson(typeAliasJson, namespaceJson.id)),
                    variables: namespaceJson.variables.map((variableJson) => migrateVariableJson(variableJson, namespaceJson.id)),
                    namespaceParentId
                };
            }
            function migrateTypeAliasJson(typeAliasJson, namespaceParentId) {
                return { ...typeAliasJson, namespaceParentId };
            }
            function migrateVariableJson(variableJson, namespaceParentId) {
                return { ...variableJson, namespaceParentId };
            }
            return {
                ...projectJson,
                typeDocJsonParserVersion: '7.3.0',
                classes: projectJson.classes.map((classJson) => migrateClassJson(classJson, null)),
                enums: projectJson.enums.map((enumJson) => migrateEnumJson(enumJson, null)),
                functions: projectJson.functions.map((functionJson) => migrateFunctionJson(functionJson, null)),
                interfaces: projectJson.interfaces.map((interfaceJson) => migrateInterfaceJson(interfaceJson, null)),
                namespaces: projectJson.namespaces.map((namespaceJson) => migrateNamespaceJson(namespaceJson, null)),
                typeAliases: projectJson.typeAliases.map((typeAliasJson) => migrateTypeAliasJson(typeAliasJson, null)),
                variables: projectJson.variables.map((variableJson) => migrateVariableJson(variableJson, null))
            };
        }
    },
    {
        from: ['v7.3.0', 'v7.3.1', 'v7.3.2', 'v7.4.0', 'v8.0.0', 'v8.0.1', 'v8.1.0', 'v8.1.1', 'v8.1.2'],
        to: 'v8.2.0',
        run(projectJson) {
            function migrateInterfaceJson(interfaceJson) {
                return {
                    ...interfaceJson,
                    properties: interfaceJson.properties.map((propertyJson) => ({
                        ...propertyJson,
                        optional: false
                    }))
                };
            }
            function migrateNamespaceJson(namespaceJson) {
                return {
                    ...namespaceJson,
                    interfaces: namespaceJson.interfaces.map(migrateInterfaceJson),
                    namespaces: namespaceJson.namespaces.map(migrateNamespaceJson)
                };
            }
            return {
                ...projectJson,
                typeDocJsonParserVersion: '8.2.0',
                interfaces: projectJson.interfaces.map(migrateInterfaceJson),
                namespaces: projectJson.namespaces.map(migrateNamespaceJson)
            };
        }
    },
    {
        from: ['v8.2.0', 'v9.0.0', 'v9.0.1', 'v10.0.0'],
        to: 'v10.1.0',
        run(projectJson) {
            return {
                ...projectJson,
                typeDocJsonParserVersion: '10.1.0',
                dependencies: {}
            };
        }
    }
];
function latest(projectJson) {
    return {
        ...projectJson,
        typeDocJsonParserVersion: ProjectParser.version
    };
}
//# sourceMappingURL=migrateToLatest.js.map