import { Parser } from '#lib/structures/Parser';
import { CommentParser, SignatureParser, SourceParser } from '#lib/structures/misc';
import { ReflectionKind, reflectionKindToString } from '#lib/types';
/**
 * Parses data from a function reflection.
 * @since 1.0.0
 */
export class FunctionParser extends Parser {
    /**
     * The namespace parent id of this function, if any.
     * @since 7.3.0
     */
    namespaceParentId;
    /**
     * The comment parser of this function.
     * @since 1.0.0
     */
    comment;
    /**
     * Whether this function is external.
     * @since 1.0.0
     */
    external;
    /**
     * The signature parsers of this function.
     * @since 1.0.0
     */
    signatures;
    constructor(data) {
        super(data);
        const { namespaceParentId, comment, external, signatures } = data;
        this.namespaceParentId = namespaceParentId;
        this.comment = comment;
        this.external = external;
        this.signatures = signatures;
    }
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this parser.
     */
    toJSON() {
        return {
            ...super.toJSON(),
            namespaceParentId: this.namespaceParentId,
            comment: this.comment.toJSON(),
            external: this.external,
            signatures: this.signatures.map((signature) => signature.toJSON())
        };
    }
    /**
     * Generates a new {@link FunctionParser} instance from the given data.
     * @since 1.0.0
     * @param reflection The reflection to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromTypeDoc(reflection, namespaceParentId) {
        const { kind, id, name, comment = { summary: [] }, sources = [], flags, signatures = [] } = reflection;
        if (kind !== ReflectionKind.Function) {
            throw new Error(`Expected Function (${ReflectionKind.Function}), but received ${reflectionKindToString(kind)} (${kind}). NAME=${name};ID=${id}`);
        }
        return new FunctionParser({
            id,
            name,
            namespaceParentId,
            comment: CommentParser.generateFromTypeDoc(comment),
            source: sources.length ? SourceParser.generateFromTypeDoc(sources[0]) : null,
            external: Boolean(flags.isExternal),
            signatures: signatures.map((signature) => SignatureParser.generateFromTypeDoc(signature))
        });
    }
    /**
     * Generates a new {@link FunctionParser} instance from the given data.
     * @param json The json to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromJson(json) {
        const { id, name, namespaceParentId, comment, source, external, signatures } = json;
        return new FunctionParser({
            id,
            name,
            namespaceParentId,
            comment: CommentParser.generateFromJson(comment),
            source: source ? SourceParser.generateFromJson(source) : null,
            external,
            signatures: signatures.map((signature) => SignatureParser.generateFromJson(signature))
        });
    }
}
//# sourceMappingURL=FunctionParser.js.map