import { FunctionParser } from '#lib/structures/FunctionParser';
import { Parser } from '#lib/structures/Parser';
import { TypeAliasParser } from '#lib/structures/TypeAliasParser';
import { VariableParser } from '#lib/structures/VariableParser';
import { ClassParser } from '#lib/structures/class-parser';
import { EnumParser } from '#lib/structures/enum-parser';
import { InterfaceParser } from '#lib/structures/interface-parser';
import { CommentParser } from '#lib/structures/misc';
import { type SearchResult } from '#lib/types';
import type { JSONOutput } from 'typedoc';
/**
 * Parses data from a namespace reflection.
 * @since 1.0.0
 */
export declare class NamespaceParser extends Parser {
    /**
     * The namespace parent id of this namespace, if any.
     * @since 7.3.0
     */
    readonly namespaceParentId: number | null;
    /**
     * The comment parser of this namespace.
     * @since 1.0.0
     */
    readonly comment: CommentParser;
    /**
     * Whether this namespace is external.
     * @since 1.0.0
     */
    readonly external: boolean;
    /**
     * The class parsers of this namespace.
     * @since 1.0.0
     */
    readonly classes: ClassParser[];
    /**
     * The enum parsers of this namespace.
     * @since 1.0.0
     */
    readonly enums: EnumParser[];
    /**
     * The function parsers of this namespace.
     * @since 1.0.0
     */
    readonly functions: FunctionParser[];
    /**
     * The interface parsers of this namespace.
     * @since 1.0.0
     */
    readonly interfaces: InterfaceParser[];
    /**
     * The namespace parsers of this namespace.
     * @since 1.0.0
     */
    readonly namespaces: NamespaceParser[];
    /**
     * The type alias parsers of this namespace.
     * @since 1.0.0
     */
    readonly typeAliases: TypeAliasParser[];
    /**
     * The variable parsers of this namespace.
     * @since 1.0.0
     */
    readonly variables: VariableParser[];
    constructor(data: NamespaceParser.Data);
    get children(): (ClassParser | EnumParser | FunctionParser | InterfaceParser | NamespaceParser | TypeAliasParser | VariableParser)[];
    /**
     * Find a parser by id.
     * @since 3.0.0
     * @param id The id of the parser to find.
     * @returns The parser with the given id, or `null` if none was found.
     */
    find(id: number): SearchResult | null;
    /**
     * Search for a parser with a given query.
     * @since 3.0.0
     * @param query The query to search with.
     * @returns An array of search results.
     */
    search(query: string): SearchResult[];
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format.
     */
    toJSON(): NamespaceParser.Json;
    /**
     * Generates a new {@link NamespaceParser} instance from the given data.
     * @since 1.0.0
     * @param reflection The reflection to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromTypeDoc(reflection: JSONOutput.DeclarationReflection, namespaceParentId: number | null): NamespaceParser;
    /**
     * Generates a new {@link NamespaceParser} instance from the given data.
     * @param json The json to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromJson(json: NamespaceParser.Json): NamespaceParser;
}
export declare namespace NamespaceParser {
    interface Data extends Parser.Data {
        /**
         * The namespace parent id of this namespace, if any.
         */
        namespaceParentId: number | null;
        /**
         * The comment parser of this namespace.
         * @since 1.0.0
         */
        comment: CommentParser;
        /**
         * Whether this namespace is external.
         * @since 1.0.0
         */
        external: boolean;
        /**
         * The class parsers of this namespace.
         * @since 1.0.0
         */
        classes: ClassParser[];
        /**
         * The enum parsers of this namespace.
         * @since 1.0.0
         */
        enums: EnumParser[];
        /**
         * The function parsers of this namespace.
         * @since 1.0.0
         */
        functions: FunctionParser[];
        /**
         * The interface parsers of this namespace.
         * @since 1.0.0
         */
        interfaces: InterfaceParser[];
        /**
         * The namespace parsers of this namespace.
         * @since 1.0.0
         */
        namespaces: NamespaceParser[];
        /**
         * The type alias parsers of this namespace.
         * @since 1.0.0
         */
        typeAliases: TypeAliasParser[];
        /**
         * The variable parsers of this namespace.
         * @since 1.0.0
         */
        variables: VariableParser[];
    }
    interface Json extends Parser.Json {
        /**
         * The namespace parent id of this namespace, if any.
         * @since 7.3.0
         */
        namespaceParentId: number | null;
        /**
         * The comment parser of this namespace.
         * @since 1.0.0
         */
        comment: CommentParser.Json;
        /**
         * Whether this namespace is external.
         * @since 1.0.0
         */
        external: boolean;
        /**
         * The class parsers of this namespace in a json compatible format.
         * @since 1.0.0
         */
        classes: ClassParser.Json[];
        /**
         * The enum parsers of this namespace in a json compatible format.
         * @since 1.0.0
         */
        enums: EnumParser.Json[];
        /**
         * The function parsers of this namespace in a json compatible format.
         * @since 1.0.0
         */
        functions: FunctionParser.Json[];
        /**
         * The interface parsers of this namespace in a json compatible format.
         * @since 1.0.0
         */
        interfaces: InterfaceParser.Json[];
        /**
         * The namespace parsers of this namespace in a json compatible format.
         * @since 1.0.0
         */
        namespaces: Json[];
        /**
         * The type alias parsers of this namespace in a json compatible format.
         * @since 1.0.0
         */
        typeAliases: TypeAliasParser.Json[];
        /**
         * The variable parsers of this namespace in a json compatible format.
         * @since 1.0.0
         */
        variables: VariableParser.Json[];
    }
}
//# sourceMappingURL=NamespaceParser.d.ts.map