import { FunctionParser } from '#lib/structures/FunctionParser';
import { Parser } from '#lib/structures/Parser';
import { TypeAliasParser } from '#lib/structures/TypeAliasParser';
import { VariableParser } from '#lib/structures/VariableParser';
import { ClassParser } from '#lib/structures/class-parser';
import { EnumParser } from '#lib/structures/enum-parser';
import { InterfaceParser } from '#lib/structures/interface-parser';
import { CommentParser, SourceParser } from '#lib/structures/misc';
import { ReflectionKind, reflectionKindToString } from '#lib/types';
/**
 * Parses data from a namespace reflection.
 * @since 1.0.0
 */
export class NamespaceParser extends Parser {
    /**
     * The namespace parent id of this namespace, if any.
     * @since 7.3.0
     */
    namespaceParentId;
    /**
     * The comment parser of this namespace.
     * @since 1.0.0
     */
    comment;
    /**
     * Whether this namespace is external.
     * @since 1.0.0
     */
    external;
    /**
     * The class parsers of this namespace.
     * @since 1.0.0
     */
    classes;
    /**
     * The enum parsers of this namespace.
     * @since 1.0.0
     */
    enums;
    /**
     * The function parsers of this namespace.
     * @since 1.0.0
     */
    functions;
    /**
     * The interface parsers of this namespace.
     * @since 1.0.0
     */
    interfaces;
    /**
     * The namespace parsers of this namespace.
     * @since 1.0.0
     */
    namespaces;
    /**
     * The type alias parsers of this namespace.
     * @since 1.0.0
     */
    typeAliases;
    /**
     * The variable parsers of this namespace.
     * @since 1.0.0
     */
    variables;
    constructor(data) {
        super(data);
        const { namespaceParentId, comment, external, classes, enums, functions, interfaces, namespaces, typeAliases, variables } = data;
        this.namespaceParentId = namespaceParentId;
        this.comment = comment;
        this.external = external;
        this.classes = classes;
        this.enums = enums;
        this.functions = functions;
        this.interfaces = interfaces;
        this.namespaces = namespaces;
        this.typeAliases = typeAliases;
        this.variables = variables;
    }
    get children() {
        return [...this.classes, ...this.enums, ...this.functions, ...this.interfaces, ...this.namespaces, ...this.typeAliases, ...this.variables];
    }
    /**
     * Find a parser by id.
     * @since 3.0.0
     * @param id The id of the parser to find.
     * @returns The parser with the given id, or `null` if none was found.
     */
    find(id) {
        for (const classParser of this.classes) {
            if (classParser.id === id) {
                return classParser;
            }
            if (classParser.construct.id === id) {
                return classParser.construct;
            }
            for (const methodParser of classParser.methods) {
                if (methodParser.id === id) {
                    return methodParser;
                }
                for (const signature of methodParser.signatures) {
                    if (signature.id === id) {
                        return signature;
                    }
                    for (const typeParameter of signature.typeParameters) {
                        if (typeParameter.id === id) {
                            return typeParameter;
                        }
                    }
                    for (const parameter of signature.parameters) {
                        if (parameter.id === id) {
                            return parameter;
                        }
                    }
                }
            }
            for (const propertyParser of classParser.properties) {
                if (propertyParser.id === id) {
                    return propertyParser;
                }
            }
        }
        for (const enumParser of this.enums) {
            if (enumParser.id === id) {
                return enumParser;
            }
            for (const propertyParser of enumParser.members) {
                if (propertyParser.id === id) {
                    return propertyParser;
                }
            }
        }
        for (const functionParser of this.functions) {
            if (functionParser.id === id) {
                return functionParser;
            }
        }
        for (const interfaceParser of this.interfaces) {
            if (interfaceParser.id === id) {
                return interfaceParser;
            }
            for (const propertyParser of interfaceParser.properties) {
                if (propertyParser.id === id) {
                    return propertyParser;
                }
            }
        }
        for (const namespaceParser of this.namespaces) {
            if (namespaceParser.id === id) {
                return namespaceParser;
            }
            const found = namespaceParser.find(id);
            if (found) {
                return found;
            }
        }
        for (const typeAliasParser of this.typeAliases) {
            if (typeAliasParser.id === id) {
                return typeAliasParser;
            }
        }
        for (const variableParser of this.variables) {
            if (variableParser.id === id) {
                return variableParser;
            }
        }
        return null;
    }
    /**
     * Search for a parser with a given query.
     * @since 3.0.0
     * @param query The query to search with.
     * @returns An array of search results.
     */
    search(query) {
        const results = [];
        const words = query
            .toLowerCase()
            .split(/(#|\.)/g)
            .filter((word) => word !== '.' && word !== '#');
        for (const classParser of this.classes) {
            if (classParser.name.toLowerCase().includes(words[0])) {
                if (words.length === 1) {
                    results.push(classParser);
                    continue;
                }
                for (const methodParser of classParser.methods) {
                    if (methodParser.name.toLowerCase().includes(words[1])) {
                        if (words.length === 2) {
                            results.push(methodParser);
                            continue;
                        }
                    }
                }
                for (const propertyParser of classParser.properties) {
                    if (propertyParser.name.toLowerCase().includes(words[1])) {
                        results.push(propertyParser);
                        continue;
                    }
                }
            }
        }
        for (const enumParser of this.enums) {
            if (enumParser.name.toLowerCase().includes(words[0])) {
                if (words.length === 1) {
                    results.push(enumParser);
                    continue;
                }
                for (const enumMemberParser of enumParser.members) {
                    if (enumMemberParser.name.toLowerCase().includes(words[1])) {
                        results.push(enumMemberParser);
                        continue;
                    }
                }
            }
        }
        for (const functionParser of this.functions) {
            if (functionParser.name.toLowerCase().includes(words[0])) {
                results.push(functionParser);
                continue;
            }
        }
        for (const interfaceParser of this.interfaces) {
            if (interfaceParser.name.toLowerCase().includes(words[0])) {
                if (words.length === 1) {
                    results.push(interfaceParser);
                    continue;
                }
                for (const propertyParser of interfaceParser.properties) {
                    if (propertyParser.name.toLowerCase().includes(words[1])) {
                        results.push(propertyParser);
                        continue;
                    }
                }
            }
        }
        for (const namespaceParser of this.namespaces) {
            if (namespaceParser.name.toLowerCase().includes(words[0])) {
                if (words.length === 1) {
                    results.push(namespaceParser);
                    continue;
                }
                const subResults = namespaceParser.search(query.substring(words[0].length));
                for (const subResult of subResults) {
                    results.push(subResult);
                }
            }
        }
        for (const typeAliasParser of this.typeAliases) {
            if (typeAliasParser.name.toLowerCase().includes(words[0])) {
                results.push(typeAliasParser);
                continue;
            }
        }
        for (const variableParser of this.variables) {
            if (variableParser.name.toLowerCase().includes(words[0])) {
                results.push(variableParser);
                continue;
            }
        }
        return results;
    }
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format.
     */
    toJSON() {
        return {
            ...super.toJSON(),
            namespaceParentId: this.namespaceParentId,
            comment: this.comment.toJSON(),
            external: this.external,
            classes: this.classes.map((parser) => parser.toJSON()),
            enums: this.enums.map((parser) => parser.toJSON()),
            functions: this.functions.map((parser) => parser.toJSON()),
            interfaces: this.interfaces.map((parser) => parser.toJSON()),
            namespaces: this.namespaces.map((parser) => parser.toJSON()),
            typeAliases: this.typeAliases.map((parser) => parser.toJSON()),
            variables: this.variables.map((parser) => parser.toJSON())
        };
    }
    /**
     * Generates a new {@link NamespaceParser} instance from the given data.
     * @since 1.0.0
     * @param reflection The reflection to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromTypeDoc(reflection, namespaceParentId) {
        const { kind, id, name, comment = { summary: [] }, sources = [], flags, children = [] } = reflection;
        if (kind !== ReflectionKind.Namespace) {
            throw new Error(`Expected Namespace (${ReflectionKind.Namespace}), but received ${reflectionKindToString(kind)} (${kind}). NAME=${name};ID=${id}`);
        }
        const classes = children.filter((child) => child.kind === ReflectionKind.Class).map((child) => ClassParser.generateFromTypeDoc(child, id));
        const enums = children.filter((child) => child.kind === ReflectionKind.Enum).map((child) => EnumParser.generateFromTypeDoc(child, id));
        const functions = children
            .filter((child) => child.kind === ReflectionKind.Function)
            .map((child) => FunctionParser.generateFromTypeDoc(child, id));
        const interfaces = children
            .filter((child) => child.kind === ReflectionKind.Interface)
            .map((child) => InterfaceParser.generateFromTypeDoc(child, id));
        const namespaces = children
            .filter((child) => child.kind === ReflectionKind.Namespace)
            .map((child) => NamespaceParser.generateFromTypeDoc(child, id));
        const typeAliases = children
            .filter((child) => child.kind === ReflectionKind.TypeAlias)
            .map((child) => TypeAliasParser.generateFromTypeDoc(child, id));
        const variables = children
            .filter((child) => child.kind === ReflectionKind.Variable)
            .map((child) => VariableParser.generateFromTypeDoc(child, id));
        return new NamespaceParser({
            id,
            name,
            namespaceParentId,
            comment: CommentParser.generateFromTypeDoc(comment),
            source: sources.length ? SourceParser.generateFromTypeDoc(sources[0]) : null,
            external: Boolean(flags.isExternal),
            classes,
            enums,
            functions,
            interfaces,
            namespaces,
            typeAliases,
            variables
        });
    }
    /**
     * Generates a new {@link NamespaceParser} instance from the given data.
     * @param json The json to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromJson(json) {
        const { id, name, namespaceParentId, comment, source, external, classes, variables, enums, functions, interfaces, namespaces, typeAliases } = json;
        return new NamespaceParser({
            id,
            name,
            namespaceParentId,
            comment: CommentParser.generateFromJson(comment),
            source: source ? SourceParser.generateFromJson(source) : null,
            external,
            classes: classes.map((json) => ClassParser.generateFromJson(json)),
            enums: enums.map((json) => EnumParser.generateFromJson(json)),
            functions: functions.map((json) => FunctionParser.generateFromJson(json)),
            interfaces: interfaces.map((json) => InterfaceParser.generateFromJson(json)),
            namespaces: namespaces.map((json) => NamespaceParser.generateFromJson(json)),
            typeAliases: typeAliases.map((json) => TypeAliasParser.generateFromJson(json)),
            variables: variables.map((json) => VariableParser.generateFromJson(json))
        });
    }
}
//# sourceMappingURL=NamespaceParser.js.map