import { Parser } from '#lib/structures/Parser';
import { CommentParser, SourceParser, TypeParameterParser } from '#lib/structures/misc';
import { TypeParser } from '#lib/structures/type-parsers';
import { ReflectionKind, reflectionKindToString } from '#lib/types';
/**
 * Parses data from a type alias reflection.
 * @since 1.0.0
 */
export class TypeAliasParser extends Parser {
    /**
     * The namespace parent id of this type alias, if any.
     * @since 7.3.0
     */
    namespaceParentId;
    /**
     * The comment parser of this type alias.
     * @since 1.0.0
     */
    comment;
    /**
     * Whether this type alias is external.
     * @since 1.0.0
     */
    external;
    /**
     * The type parameters of this type alias.
     * @since 1.0.0
     */
    typeParameters;
    /**
     * The type of this type alias.
     * @since 1.0.0
     */
    type;
    constructor(data) {
        super(data);
        const { namespaceParentId, comment, external, typeParameters, type } = data;
        this.namespaceParentId = namespaceParentId;
        this.comment = comment;
        this.external = external;
        this.typeParameters = typeParameters;
        this.type = type;
    }
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this parser.
     */
    toJSON() {
        return {
            ...super.toJSON(),
            namespaceParentId: this.namespaceParentId,
            comment: this.comment.toJSON(),
            external: this.external,
            typeParameters: this.typeParameters.map((typeParameter) => typeParameter.toJSON()),
            type: this.type.toJSON()
        };
    }
    /**
     * Generates a new {@link TypeAliasParser} instance from the given data.
     * @since 1.0.0
     * @param reflection The reflection to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromTypeDoc(reflection, namespaceParentId) {
        const { kind, id, name, comment = { summary: [] }, sources = [], flags, type, typeParameters = [] } = reflection;
        if (kind !== ReflectionKind.TypeAlias) {
            throw new Error(`Expected TypeAlias (${ReflectionKind.TypeAlias}), but received ${reflectionKindToString(kind)} (${kind}). NAME=${name};ID=${id}`);
        }
        return new TypeAliasParser({
            id,
            name,
            namespaceParentId,
            comment: CommentParser.generateFromTypeDoc(comment),
            source: sources.length ? SourceParser.generateFromTypeDoc(sources[0]) : null,
            external: Boolean(flags.isExternal),
            typeParameters: typeParameters.map((typeParameter) => TypeParameterParser.generateFromTypeDoc(typeParameter)),
            type: TypeParser.generateFromTypeDoc(type)
        });
    }
    /**
     * Generates a new {@link TypeAliasParser} instance from the given data.
     * @param json The json to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromJson(json) {
        const { id, name, namespaceParentId, comment, source, external, typeParameters, type } = json;
        return new TypeAliasParser({
            id,
            name,
            namespaceParentId,
            comment: CommentParser.generateFromJson(comment),
            source: source ? SourceParser.generateFromJson(source) : null,
            external,
            typeParameters: typeParameters.map((typeParameter) => TypeParameterParser.generateFromJson(typeParameter)),
            type: TypeParser.generateFromJson(type)
        });
    }
}
//# sourceMappingURL=TypeAliasParser.js.map