import { Parser } from '#lib/structures/Parser';
import { CommentParser, SourceParser } from '#lib/structures/misc';
import { TypeParser } from '#lib/structures/type-parsers';
import { ReflectionKind, reflectionKindToString } from '#lib/types';
/**
 * Parses data from a variable reflection.
 * @since 1.0.0
 */
export class VariableParser extends Parser {
    /**
     * The namespace parent id of this variable, if any.
     * @since 7.3.0
     */
    namespaceParentId;
    /**
     * The comment parser of this variable.
     * @since 1.0.0
     */
    comment;
    /**
     * Whether this variable is external.
     * @since 1.0.0
     */
    external;
    /**
     * The type of this variable.
     * @since 1.0.0
     */
    type;
    /**
     * The value of this variable.
     * @since 1.0.0
     */
    value;
    constructor(data) {
        super(data);
        const { namespaceParentId, comment, external, type, value } = data;
        this.namespaceParentId = namespaceParentId;
        this.comment = comment;
        this.external = external;
        this.type = type;
        this.value = value;
    }
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this parser.
     */
    toJSON() {
        return {
            ...super.toJSON(),
            namespaceParentId: this.namespaceParentId,
            comment: this.comment.toJSON(),
            external: this.external,
            type: this.type.toJSON(),
            value: this.value
        };
    }
    /**
     * Generates a new {@link VariableParser} instance from the given data.
     * @since 1.0.0
     * @param reflection The reflection to generate the parser from.
     * @param namespaceParentId The this parser belongs to.
     * @returns The generated parser.
     */
    static generateFromTypeDoc(reflection, namespaceParentId) {
        const { kind, id, name, comment = { summary: [] }, sources = [], flags, type, defaultValue } = reflection;
        if (kind !== ReflectionKind.Variable) {
            throw new Error(`Expected Variable (${ReflectionKind.Variable}), but received ${reflectionKindToString(kind)} (${kind}). NAME=${name};ID=${id}`);
        }
        return new VariableParser({
            id,
            name,
            namespaceParentId,
            comment: CommentParser.generateFromTypeDoc(comment),
            source: sources.length ? SourceParser.generateFromTypeDoc(sources[0]) : null,
            external: Boolean(flags.isExternal),
            type: TypeParser.generateFromTypeDoc(type),
            value: defaultValue
        });
    }
    /**
     * Generates a new {@link VariableParser} instance from the given data.
     * @param json The json to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromJson(json) {
        const { id, name, namespaceParentId, comment, source, external, type, value } = json;
        return new VariableParser({
            id,
            name,
            namespaceParentId,
            comment: CommentParser.generateFromJson(comment),
            source: source ? SourceParser.generateFromJson(source) : null,
            external,
            type: TypeParser.generateFromJson(type),
            value
        });
    }
}
//# sourceMappingURL=VariableParser.js.map