import { Parser } from '#lib/structures/Parser';
import { ClassParser } from '#lib/structures/class-parser';
import { CommentParser, ParameterParser, SourceParser } from '#lib/structures/misc';
import { ReflectionKind, reflectionKindToString } from '#lib/types';
export class ClassConstructorParser extends Parser {
    /**
     * The comment parser of this constructor.
     * @since 1.0.0
     */
    comment;
    /**
     * The id of the parent class parser.
     * @since 4.0.0
     */
    parentId;
    /**
     * The accessibility of this constructor.
     * @since 7.0.0
     */
    accessibility;
    /**
     * The parameter parsers of this constructor.
     * @since 1.0.0
     */
    parameters;
    constructor(data) {
        super(data);
        const { comment, parentId, accessibility, parameters } = data;
        this.comment = comment;
        this.parentId = parentId;
        this.accessibility = accessibility;
        this.parameters = parameters;
    }
    /**
     * Whether or not this constructor has a public accessibility.
     * @since 7.0.0
     * @returns The validation boolean.
     */
    isPublic() {
        return this.accessibility === ClassParser.Accessibility.Public;
    }
    /**
     * Whether or not this constructor has a protected accessibility.
     * @since 7.0.0
     * @returns The validation boolean.
     */
    isProtected() {
        return this.accessibility === ClassParser.Accessibility.Protected;
    }
    /**
     * Whether or not this constructor has a private accessibility.
     * @since 7.0.0
     * @returns The validation boolean.
     */
    isPrivate() {
        return this.accessibility === ClassParser.Accessibility.Private;
    }
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this parser.
     */
    toJSON() {
        return {
            ...super.toJSON(),
            comment: this.comment.toJSON(),
            parentId: this.parentId,
            accessibility: this.accessibility,
            parameters: this.parameters.map((parameter) => parameter.toJSON())
        };
    }
    /**
     * Generates a new {@link ClassConstructorParser} instance from the given data.
     * @since 1.0.0
     * @param reflection The reflection to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromTypeDoc(reflection, parentId) {
        const { kind, id, name, comment = { summary: [] }, sources = [], flags, signatures = [] } = reflection;
        if (kind !== ReflectionKind.Constructor) {
            throw new Error(`Expected Constructor (${ReflectionKind.Constructor}), but received ${reflectionKindToString(kind)} (${kind}). NAME=${name};ID=${id}`);
        }
        const signature = signatures.find((signature) => signature.kind === ReflectionKind.ConstructorSignature);
        if (signature === undefined) {
            throw new Error(`Expected Constructor (${ReflectionKind.Constructor}) with a signature, but there was none`);
        }
        const { parameters = [] } = signature;
        return new ClassConstructorParser({
            id,
            name,
            comment: CommentParser.generateFromTypeDoc(comment),
            source: sources.length ? SourceParser.generateFromTypeDoc(sources[0]) : null,
            parentId,
            accessibility: flags.isPrivate
                ? ClassParser.Accessibility.Private
                : flags.isProtected
                    ? ClassParser.Accessibility.Protected
                    : ClassParser.Accessibility.Public,
            parameters: parameters.map((parameter) => ParameterParser.generateFromTypeDoc(parameter))
        });
    }
    /**
     * Generates a new {@link ClassConstructorParser} instance from the given data.
     * @param json The json to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromJson(json) {
        const { id, name, comment, source, parentId, accessibility, parameters } = json;
        return new ClassConstructorParser({
            id,
            name,
            comment: CommentParser.generateFromJson(comment),
            source: source ? SourceParser.generateFromJson(source) : null,
            parentId,
            accessibility,
            parameters: parameters.map((parameter) => ParameterParser.generateFromJson(parameter))
        });
    }
}
//# sourceMappingURL=ClassConstructorParser.js.map