import { Parser } from '#lib/structures/Parser';
import { ClassParser } from '#lib/structures/class-parser';
import { SignatureParser, SourceParser } from '#lib/structures/misc';
import { ReflectionKind, reflectionKindToString } from '#lib/types';
/**
 * Parses data from a class method reflection.
 * @since 1.0.0
 */
export class ClassMethodParser extends Parser {
    /**
     * The id of the parent class parser.
     * @since 4.0.0
     */
    parentId;
    /**
     * The accessibility of this method.
     * @since 1.0.0
     */
    accessibility;
    /**
     * Whether this method is abstract.
     * @since 1.0.0
     */
    abstract;
    /**
     * Whether this method is static.
     * @since 1.0.0
     */
    static;
    /**
     * The signature parsers of this method.
     * @since 1.0.0
     */
    signatures;
    constructor(data) {
        super(data);
        const { parentId, accessibility, abstract, static: _static, signatures } = data;
        this.parentId = parentId;
        this.accessibility = accessibility;
        this.abstract = abstract;
        this.static = _static;
        this.signatures = signatures;
    }
    /**
     * Whether or not this method has a public accessibility.
     * @since 7.0.0
     * @returns The validation boolean.
     */
    isPublic() {
        return this.accessibility === ClassParser.Accessibility.Public;
    }
    /**
     * Whether or not this method has a protected accessibility.
     * @since 7.0.0
     * @returns The validation boolean.
     */
    isProtected() {
        return this.accessibility === ClassParser.Accessibility.Protected;
    }
    /**
     * Whether or not this method has a private accessibility.
     * @since 7.0.0
     * @returns The validation boolean.
     */
    isPrivate() {
        return this.accessibility === ClassParser.Accessibility.Private;
    }
    /**
     * Convert this parser to a JSON compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this parser.
     */
    toJSON() {
        return {
            ...super.toJSON(),
            parentId: this.parentId,
            accessibility: this.accessibility,
            abstract: this.abstract,
            static: this.static,
            signatures: this.signatures.map((signature) => signature.toJSON())
        };
    }
    /**
     * Generates a new {@link ClassMethodParser} instance from the given data.
     * @since 1.0.0
     * @param reflection The reflection to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromTypeDoc(reflection, parentId) {
        const { kind, id, name, sources = [], flags, signatures = [] } = reflection;
        if (kind !== ReflectionKind.Method) {
            throw new Error(`Expected Method (${ReflectionKind.Method}), but received ${reflectionKindToString(kind)} (${kind}). NAME=${name};ID=${id}`);
        }
        return new ClassMethodParser({
            id,
            name,
            source: sources.length ? SourceParser.generateFromTypeDoc(sources[0]) : null,
            parentId,
            accessibility: flags.isPrivate
                ? ClassParser.Accessibility.Private
                : flags.isProtected
                    ? ClassParser.Accessibility.Protected
                    : ClassParser.Accessibility.Public,
            abstract: Boolean(flags.isAbstract),
            static: Boolean(flags.isStatic),
            signatures: signatures.map((signature) => SignatureParser.generateFromTypeDoc(signature))
        });
    }
    /**
     * Generates a new {@link ClassMethodParser} instance from the given data.
     * @param json The json to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromJson(json) {
        const { id, name, source, parentId, accessibility, abstract, static: _static, signatures } = json;
        return new ClassMethodParser({
            id,
            name,
            source: source ? SourceParser.generateFromJson(source) : null,
            parentId,
            accessibility,
            abstract,
            static: _static,
            signatures: signatures.map((signature) => SignatureParser.generateFromJson(signature))
        });
    }
}
//# sourceMappingURL=ClassMethodParser.js.map