import { Parser } from '#lib/structures/Parser';
import { ClassConstructorParser, ClassMethodParser, ClassPropertyParser } from '#lib/structures/class-parser';
import { CommentParser, TypeParameterParser } from '#lib/structures/misc';
import { TypeParser } from '#lib/structures/type-parsers';
import type { JSONOutput } from 'typedoc';
/**
 * Parses data from a class reflection.
 * @since 1.0.0
 */
export declare class ClassParser extends Parser {
    /**
     * The namespace parent id of this class, if any.
     * @since 7.3.0
     */
    readonly namespaceParentId: number | null;
    /**
     * The comment parser of this class.
     * @since 1.0.0
     */
    readonly comment: CommentParser;
    /**
     * Whether this class is external.
     * @since 1.0.0
     */
    readonly external: boolean;
    /**
     * Whether this class is abstract.
     * @since 1.0.0
     */
    readonly abstract: boolean;
    /**
     * The `extends` type of this class.
     * @since 1.0.0
     */
    readonly extendsType: TypeParser | null;
    /**
     * The `implements` type of this class.
     * @since 1.0.0
     */
    readonly implementsType: TypeParser[];
    /**
     * The type parameter parsers of this class.
     * @since 6.0.0
     */
    readonly typeParameters: TypeParameterParser[];
    /**
     * The constructor parser of this class.
     * @since 1.0.0
     */
    readonly construct: ClassConstructorParser;
    /**
     * The property parsers of this class.
     * @since 1.0.0
     */
    readonly properties: ClassPropertyParser[];
    /**
     * The method parsers of this class.
     * @since 1.0.0
     */
    readonly methods: ClassMethodParser[];
    constructor(data: ClassParser.Data);
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this parser.
     */
    toJSON(): ClassParser.Json;
    /**
     * Generates a new {@link ClassParser} instance from the given data.
     * @since 1.0.0
     * @param reflection The reflection to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromTypeDoc(reflection: JSONOutput.DeclarationReflection, namespaceParentId: number | null): ClassParser;
    /**
     * Generates a new {@link ClassParser} instance from the given data.
     * @param json The json to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromJson(json: ClassParser.Json): ClassParser;
}
export declare namespace ClassParser {
    interface Data extends Parser.Data {
        /**
         * The namespace parent id of this class, if any.
         * @since 7.3.0
         */
        namespaceParentId: number | null;
        /**
         * The comment parser of this class.
         * @since 1.0.0
         */
        comment: CommentParser;
        /**
         * Whether this class is external.
         * @since 1.0.0
         */
        external: boolean;
        /**
         * Whether this class is abstract.
         * @since 1.0.0
         */
        abstract: boolean;
        /**
         * The `extends` type of this class.
         * @since 1.0.0
         */
        extendsType: TypeParser | null;
        /**
         * The `implements` type of this class.
         * @since 1.0.0
         */
        implementsType: TypeParser[];
        /**
         * The type parameter parsers of this class.
         * @since 6.0.0
         */
        typeParameters: TypeParameterParser[];
        /**
         * The constructor parser of this class.
         * @since 1.0.0
         */
        construct: ClassConstructorParser;
        /**
         * The property parsers of this class.
         * @since 1.0.0
         */
        properties: ClassPropertyParser[];
        /**
         * The method parsers of this class.
         * @since 1.0.0
         */
        methods: ClassMethodParser[];
    }
    interface Json extends Parser.Json {
        /**
         * The namespace parent id of this class, if any.
         * @since 7.3.0
         */
        namespaceParentId: number | null;
        /**
         * The comment parser of this class.
         * @since 1.0.0
         */
        comment: CommentParser.Json;
        /**
         * Whether this class is external.
         * @since 1.0.0
         */
        external: boolean;
        /**
         * Whether this class is abstract.
         * @since 1.0.0
         */
        abstract: boolean;
        /**
         * The `extends` type of this class in a json compatible format.
         * @since 1.0.0
         */
        extendsType: TypeParser.Json | null;
        /**
         * The `implements` type of this class in a json compatible format.
         * @since 1.0.0
         */
        implementsType: TypeParser.Json[];
        /**
         * The type parameter parsers of this class in a json compatible format.
         * @since 6.0.0
         */
        typeParameters: TypeParameterParser.Json[];
        /**
         * The constructor parser of this class in a json compatible format.
         * @since 1.0.0
         */
        construct: ClassConstructorParser.Json;
        /**
         * The property parsers of this class in a json compatible format.
         * @since 1.0.0
         */
        properties: ClassPropertyParser.Json[];
        /**
         * The method parsers of this class in a json compatible format.
         * @since 1.0.0
         */
        methods: ClassMethodParser.Json[];
    }
    /**
     * The accessibility types of a class.
     * @since 1.0.0
     */
    enum Accessibility {
        Public = "public",
        Protected = "protected",
        Private = "private"
    }
}
//# sourceMappingURL=ClassParser.d.ts.map