import { Parser } from '#lib/structures/Parser';
import { ClassConstructorParser, ClassMethodParser, ClassPropertyParser } from '#lib/structures/class-parser';
import { CommentParser, SourceParser, TypeParameterParser } from '#lib/structures/misc';
import { TypeParser } from '#lib/structures/type-parsers';
import { ReflectionKind, reflectionKindToString } from '#lib/types';
/**
 * Parses data from a class reflection.
 * @since 1.0.0
 */
export class ClassParser extends Parser {
    /**
     * The namespace parent id of this class, if any.
     * @since 7.3.0
     */
    namespaceParentId;
    /**
     * The comment parser of this class.
     * @since 1.0.0
     */
    comment;
    /**
     * Whether this class is external.
     * @since 1.0.0
     */
    external;
    /**
     * Whether this class is abstract.
     * @since 1.0.0
     */
    abstract;
    /**
     * The `extends` type of this class.
     * @since 1.0.0
     */
    extendsType;
    /**
     * The `implements` type of this class.
     * @since 1.0.0
     */
    implementsType;
    /**
     * The type parameter parsers of this class.
     * @since 6.0.0
     */
    typeParameters;
    /**
     * The constructor parser of this class.
     * @since 1.0.0
     */
    construct;
    /**
     * The property parsers of this class.
     * @since 1.0.0
     */
    properties;
    /**
     * The method parsers of this class.
     * @since 1.0.0
     */
    methods;
    constructor(data) {
        super(data);
        const { namespaceParentId, comment, external, abstract, extendsType, implementsType, typeParameters, construct, properties, methods } = data;
        this.namespaceParentId = namespaceParentId;
        this.comment = comment;
        this.external = external;
        this.abstract = abstract;
        this.extendsType = extendsType;
        this.implementsType = implementsType;
        this.typeParameters = typeParameters;
        this.construct = construct;
        this.properties = properties;
        this.methods = methods;
    }
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this parser.
     */
    toJSON() {
        return {
            ...super.toJSON(),
            namespaceParentId: this.namespaceParentId,
            comment: this.comment.toJSON(),
            external: this.external,
            abstract: this.abstract,
            extendsType: this.extendsType ? this.extendsType.toJSON() : null,
            implementsType: this.implementsType.map((implementsType) => implementsType.toJSON()),
            typeParameters: this.typeParameters.map((typeParameter) => typeParameter.toJSON()),
            construct: this.construct.toJSON(),
            properties: this.properties,
            methods: this.methods
        };
    }
    /**
     * Generates a new {@link ClassParser} instance from the given data.
     * @since 1.0.0
     * @param reflection The reflection to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromTypeDoc(reflection, namespaceParentId) {
        const { kind, id, name, comment = { summary: [] }, sources = [], flags, children = [], extendedTypes = [], implementedTypes = [], typeParameters = [] } = reflection;
        if (kind !== ReflectionKind.Class) {
            throw new Error(`Expected Class (${ReflectionKind.Class}), but received ${reflectionKindToString(kind)} (${kind}) NAME=${name};ID=${id}`);
        }
        const construct = children.find((child) => child.kind === ReflectionKind.Constructor);
        if (construct === undefined) {
            throw new Error(`Expected Class (${ReflectionKind.Class}) with a constructor, but there was none. NAME=${name};ID=${id}`);
        }
        const properties = children
            .filter((child) => child.kind === ReflectionKind.Property || (child.kind === ReflectionKind.Accessor && child.getSignature))
            .map((child) => ClassPropertyParser.generateFromTypeDoc(child, id));
        const methods = children.filter((child) => child.kind === ReflectionKind.Method).map((child) => ClassMethodParser.generateFromTypeDoc(child, id));
        return new ClassParser({
            id,
            name,
            namespaceParentId,
            comment: CommentParser.generateFromTypeDoc(comment),
            source: sources.length ? SourceParser.generateFromTypeDoc(sources[0]) : null,
            external: Boolean(flags.isExternal),
            abstract: Boolean(flags.isAbstract),
            extendsType: extendedTypes.length ? TypeParser.generateFromTypeDoc(extendedTypes[0]) : null,
            implementsType: implementedTypes.map((implementedType) => TypeParser.generateFromTypeDoc(implementedType)),
            typeParameters: typeParameters.map((typeParameter) => TypeParameterParser.generateFromTypeDoc(typeParameter)),
            construct: ClassConstructorParser.generateFromTypeDoc(construct, id),
            properties,
            methods
        });
    }
    /**
     * Generates a new {@link ClassParser} instance from the given data.
     * @param json The json to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromJson(json) {
        const { id, name, namespaceParentId, comment, source, external, abstract, extendsType, implementsType, typeParameters, construct, properties, methods } = json;
        return new ClassParser({
            id,
            name,
            namespaceParentId,
            comment: CommentParser.generateFromJson(comment),
            source: source ? SourceParser.generateFromJson(source) : null,
            external,
            abstract,
            extendsType: extendsType ? TypeParser.generateFromJson(extendsType) : null,
            implementsType: implementsType.map((implementedType) => TypeParser.generateFromJson(implementedType)),
            typeParameters: typeParameters.map((typeParameter) => TypeParameterParser.generateFromJson(typeParameter)),
            construct: ClassConstructorParser.generateFromJson(construct),
            properties: properties.map((property) => ClassPropertyParser.generateFromJson(property)),
            methods: methods.map((method) => ClassMethodParser.generateFromJson(method))
        });
    }
}
(function (ClassParser) {
    /**
     * The accessibility types of a class.
     * @since 1.0.0
     */
    let Accessibility;
    (function (Accessibility) {
        Accessibility["Public"] = "public";
        Accessibility["Protected"] = "protected";
        Accessibility["Private"] = "private";
    })(Accessibility = ClassParser.Accessibility || (ClassParser.Accessibility = {}));
})(ClassParser || (ClassParser = {}));
//# sourceMappingURL=ClassParser.js.map