import { Parser } from '#lib/structures/Parser';
import { ClassParser } from '#lib/structures/class-parser';
import { CommentParser, SourceParser } from '#lib/structures/misc';
import { TypeParser } from '#lib/structures/type-parsers';
import { ReflectionKind, reflectionKindToString } from '#lib/types';
/**
 * Parses data from a class property reflection.
 * @since 1.0.0
 */
export class ClassPropertyParser extends Parser {
    /**
     * The comment parser of this property.
     * @since 1.0.0
     */
    comment;
    /**
     * The id of the parent class parser.
     * @since 4.0.0
     */
    parentId;
    /**
     * The accessibility of this property.
     * @since 1.0.0
     */
    accessibility;
    /**
     * Whether this property is abstract.
     * @since 1.0.0
     */
    abstract;
    /**
     * Whether this property is static.
     * @since 1.0.0
     */
    static;
    /**
     * Whether this property is readonly.
     * @since 1.0.0
     */
    readonly;
    /**
     * Whether this property is optional.
     * @since 1.0.0
     */
    optional;
    /**
     * The type parser of this property.
     * @since 1.0.0
     */
    type;
    constructor(data) {
        super(data);
        const { comment, parentId, accessibility, abstract, static: _static, readonly, optional, type } = data;
        this.comment = comment;
        this.parentId = parentId;
        this.accessibility = accessibility;
        this.abstract = abstract;
        this.static = _static;
        this.readonly = readonly;
        this.optional = optional;
        this.type = type;
    }
    /**
     * Whether or not this property has a public accessibility.
     * @since 7.0.0
     * @returns The validation boolean.
     */
    isPublic() {
        return this.accessibility === ClassParser.Accessibility.Public;
    }
    /**
     * Whether or not this property has a protected accessibility.
     * @since 7.0.0
     * @returns The validation boolean.
     */
    isProtected() {
        return this.accessibility === ClassParser.Accessibility.Protected;
    }
    /**
     * Whether or not this property has a private accessibility.
     * @since 7.0.0
     * @returns The validation boolean.
     */
    isPrivate() {
        return this.accessibility === ClassParser.Accessibility.Private;
    }
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this parser.
     */
    toJSON() {
        return {
            ...super.toJSON(),
            comment: this.comment.toJSON(),
            parentId: this.parentId,
            accessibility: this.accessibility,
            abstract: this.abstract,
            static: this.static,
            readonly: this.readonly,
            optional: this.optional,
            type: this.type.toJSON()
        };
    }
    /**
     * Generates a new {@link ClassPropertyParser} instance from the given data.
     * @since 1.0.0
     * @param reflection The reflection to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromTypeDoc(reflection, parentId) {
        const { kind, id, name, comment = { summary: [] }, sources = [], type, flags, getSignature } = reflection;
        if (kind !== ReflectionKind.Property && kind !== ReflectionKind.Accessor) {
            throw new Error(`Expected Property (${ReflectionKind.Property}) or Accessor (${ReflectionKind.Accessor}), but received ${reflectionKindToString(kind)} (${kind}). NAME=${name};ID=${id}`);
        }
        if (kind === ReflectionKind.Accessor) {
            if (getSignature === undefined) {
                throw new Error(`Expected Accessor (${ReflectionKind.Accessor}) with a getter, but there was none`);
            }
            const { id, name, comment = { summary: [] }, type, flags } = getSignature;
            return new ClassPropertyParser({
                id,
                name,
                comment: CommentParser.generateFromTypeDoc(comment),
                source: sources.length ? SourceParser.generateFromTypeDoc(sources[0]) : null,
                parentId,
                accessibility: flags.isPrivate
                    ? ClassParser.Accessibility.Private
                    : flags.isProtected
                        ? ClassParser.Accessibility.Protected
                        : ClassParser.Accessibility.Public,
                abstract: Boolean(flags.isAbstract),
                static: Boolean(flags.isStatic),
                readonly: Boolean(flags.isReadonly),
                optional: Boolean(flags.isOptional),
                type: TypeParser.generateFromTypeDoc(type)
            });
        }
        return new ClassPropertyParser({
            id,
            name,
            comment: CommentParser.generateFromTypeDoc(comment),
            source: sources.length ? SourceParser.generateFromTypeDoc(sources[0]) : null,
            parentId,
            accessibility: flags.isPrivate
                ? ClassParser.Accessibility.Private
                : flags.isProtected
                    ? ClassParser.Accessibility.Protected
                    : ClassParser.Accessibility.Public,
            abstract: Boolean(flags.isAbstract),
            static: Boolean(flags.isStatic),
            readonly: Boolean(flags.isReadonly),
            optional: Boolean(flags.isOptional),
            type: TypeParser.generateFromTypeDoc(type)
        });
    }
    /**
     * Generates a new {@link ClassPropertyParser} instance from the given data.
     * @param json The json to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromJson(json) {
        const { id, name, comment, source, parentId, accessibility, abstract, static: _static, readonly, optional, type } = json;
        return new ClassPropertyParser({
            id,
            name,
            comment: CommentParser.generateFromJson(comment),
            source: source ? SourceParser.generateFromJson(source) : null,
            parentId,
            accessibility,
            abstract,
            static: _static,
            readonly,
            optional,
            type: TypeParser.generateFromJson(type)
        });
    }
}
//# sourceMappingURL=ClassPropertyParser.js.map