import { Parser } from '#lib/structures/Parser';
import { CommentParser, SourceParser } from '#lib/structures/misc';
import { TypeParser } from '#lib/structures/type-parsers';
import { ReflectionKind, reflectionKindToString } from '#lib/types';
/**
 * Parses data from an enum property reflection.
 * @since 1.0.0
 */
export class EnumMemberParser extends Parser {
    /**
     * The comment parser of this property.
     * @since 1.0.0
     */
    comment;
    /**
     * The id of the parent enum parser.
     * @since 4.0.0
     */
    parentId;
    /**
     * The value of this enum property.
     * @since 1.0.0
     */
    value;
    constructor(data) {
        super(data);
        const { comment, parentId, value } = data;
        this.comment = comment;
        this.parentId = parentId;
        this.value = value;
    }
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this parser.
     */
    toJSON() {
        return {
            ...super.toJSON(),
            comment: this.comment.toJSON(),
            parentId: this.parentId,
            value: this.value
        };
    }
    /**
     * Generates a new {@link EnumMemberParser} instance from the given data.
     * @since 1.0.0
     * @param reflection The reflection to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromTypeDoc(reflection, parentId) {
        const { kind, id, name, comment = { summary: [] }, sources = [], type } = reflection;
        if (kind !== ReflectionKind.EnumMember) {
            throw new Error(`Expected EnumMember (${ReflectionKind.EnumMember}), but received ${reflectionKindToString(kind)} (${kind}). NAME=${name};ID=${id}`);
        }
        return new EnumMemberParser({
            id,
            name,
            comment: CommentParser.generateFromTypeDoc(comment),
            source: sources.length ? SourceParser.generateFromTypeDoc(sources[0]) : null,
            parentId,
            value: TypeParser.generateFromTypeDoc(type).toString()
        });
    }
    /**
     * Generates a new {@link EnumMemberParser} instance from the given data.
     * @param json The json to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromJson(json) {
        const { id, name, comment, source, parentId, value } = json;
        return new EnumMemberParser({
            id,
            name,
            comment: CommentParser.generateFromJson(comment),
            source: source ? SourceParser.generateFromJson(source) : null,
            parentId,
            value
        });
    }
}
//# sourceMappingURL=EnumMemberParser.js.map