import { Parser } from '#lib/structures/Parser';
import { EnumMemberParser } from '#lib/structures/enum-parser';
import { CommentParser, SourceParser } from '#lib/structures/misc';
import { ReflectionKind, reflectionKindToString } from '#lib/types';
/**
 * Parses data from an enum reflection.
 * @since 1.0.0
 */
export class EnumParser extends Parser {
    /**
     * The namespace parent id of this enum, if any.
     * @since 7.3.0
     */
    namespaceParentId;
    /**
     * The comment parser of this enum.
     * @since 1.0.0
     */
    comment;
    /**
     * Whether this enum is external.
     * @since 1.0.0
     */
    external;
    /**
     * The property parsers of this enum.
     * @since 1.0.0
     */
    members;
    constructor(data) {
        super(data);
        const { namespaceParentId, comment, external, members } = data;
        this.namespaceParentId = namespaceParentId;
        this.comment = comment;
        this.external = external;
        this.members = members;
    }
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this parser.
     */
    toJSON() {
        return {
            ...super.toJSON(),
            namespaceParentId: this.namespaceParentId,
            comment: this.comment.toJSON(),
            external: this.external,
            members: this.members.map((member) => member.toJSON())
        };
    }
    /**
     * Generates a new {@link EnumParser} instance from the given Json data.
     * @since 1.0.0
     * @param reflection The reflection to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromTypeDoc(reflection, namespaceParentId) {
        const { kind, id, name, comment = { summary: [] }, sources = [], flags, children = [] } = reflection;
        if (kind !== ReflectionKind.Enum) {
            throw new Error(`Expected Enum (${ReflectionKind.Enum}), but received ${reflectionKindToString(kind)} (${kind}). NAME=${name};ID=${id}`);
        }
        const members = children
            .filter((child) => child.kind === ReflectionKind.EnumMember)
            .map((child) => EnumMemberParser.generateFromTypeDoc(child, id));
        return new EnumParser({
            id,
            name,
            namespaceParentId,
            comment: CommentParser.generateFromTypeDoc(comment),
            source: sources.length ? SourceParser.generateFromTypeDoc(sources[0]) : null,
            external: Boolean(flags.isExternal),
            members
        });
    }
    /**
     * Generates a new {@link EnumParser} instance from the given data.
     * @param json The json to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromJson(json) {
        const { id, name, namespaceParentId, comment, source, external, members } = json;
        return new EnumParser({
            id,
            name,
            namespaceParentId,
            comment: CommentParser.generateFromJson(comment),
            source: source ? SourceParser.generateFromJson(source) : null,
            external,
            members: members.map((property) => EnumMemberParser.generateFromJson(property))
        });
    }
}
//# sourceMappingURL=EnumParser.js.map