import { Parser } from '#lib/structures/Parser';
import { InterfaceMethodParser, InterfacePropertyParser } from '#lib/structures/interface-parser';
import { CommentParser, SourceParser, TypeParameterParser } from '#lib/structures/misc';
import { ReflectionKind, reflectionKindToString } from '#lib/types';
/**
 * Parses data from an interface reflection.
 * @since 1.0.0
 */
export class InterfaceParser extends Parser {
    /**
     * The namespace parent id of this interface, if any.
     * @since 7.3.0
     */
    namespaceParentId;
    /**
     * The comment parser of this interface.
     * @since 1.0.0
     */
    comment;
    /**
     * Whether this interface is external.
     * @since 1.0.0
     */
    external;
    /**
     * The type parameters of this interface.
     * @since 7.0.0
     */
    typeParameters;
    /**
     * The property parsers of this interface.
     * @since 1.0.0
     */
    properties;
    /**
     * The method parsers of this interface.
     * @since 3.1.0
     */
    methods;
    constructor(data) {
        super(data);
        const { namespaceParentId, comment, external, typeParameters, properties, methods } = data;
        this.namespaceParentId = namespaceParentId;
        this.comment = comment;
        this.external = external;
        this.typeParameters = typeParameters;
        this.properties = properties;
        this.methods = methods;
    }
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this parser.
     */
    toJSON() {
        return {
            ...super.toJSON(),
            namespaceParentId: this.namespaceParentId,
            comment: this.comment.toJSON(),
            external: this.external,
            typeParameters: this.typeParameters.map((typeParameter) => typeParameter.toJSON()),
            properties: this.properties.map((parser) => parser.toJSON()),
            methods: this.methods.map((parser) => parser.toJSON())
        };
    }
    /**
     * Generates a new {@link InterfaceParser} instance from the given data.
     * @since 1.0.0
     * @param reflection The reflection to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromTypeDoc(reflection, namespaceParentId) {
        const { kind, id, name, comment = { summary: [] }, sources = [], flags, typeParameters = [], children = [] } = reflection;
        if (kind !== ReflectionKind.Interface) {
            throw new Error(`Expected Interface (${ReflectionKind.Interface}), but received ${reflectionKindToString(kind)} (${kind}). NAME=${name};ID=${id}`);
        }
        const properties = children
            .filter((child) => child.kind === ReflectionKind.Property)
            .map((child) => InterfacePropertyParser.generateFromTypeDoc(child, id));
        const methods = children
            .filter((child) => child.kind === ReflectionKind.Method)
            .map((child) => InterfaceMethodParser.generateFromTypeDoc(child, id));
        return new InterfaceParser({
            id,
            name,
            namespaceParentId,
            comment: CommentParser.generateFromTypeDoc(comment),
            source: sources.length ? SourceParser.generateFromTypeDoc(sources[0]) : null,
            external: Boolean(flags.isExternal),
            typeParameters: typeParameters.map((typeParameter) => TypeParameterParser.generateFromTypeDoc(typeParameter)),
            properties,
            methods
        });
    }
    /**
     * Generates a new {@link InterfaceParser} instance from the given data.
     * @param json The json to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromJson(json) {
        const { id, name, namespaceParentId, comment, source, external, typeParameters, properties, methods } = json;
        return new InterfaceParser({
            id,
            name,
            namespaceParentId,
            comment: CommentParser.generateFromJson(comment),
            source: source ? SourceParser.generateFromJson(source) : null,
            external,
            typeParameters: typeParameters.map((typeParameter) => TypeParameterParser.generateFromJson(typeParameter)),
            properties: properties.map((parser) => InterfacePropertyParser.generateFromJson(parser)),
            methods: methods.map((parser) => InterfaceMethodParser.generateFromJson(parser))
        });
    }
}
//# sourceMappingURL=InterfaceParser.js.map