import { CommentParser, ParameterParser, TypeParameterParser } from '#lib/structures/misc';
import { TypeParser } from '#lib/structures/type-parsers';
import { ReflectionKind, reflectionKindToString } from '#lib/types';
/**
 * Parses data from a signature reflection.
 * @since 1.0.0
 */
export class SignatureParser {
    /**
     * The identifier of this parser.
     * @since 1.0.0
     */
    id;
    /**
     * The name of this signature.
     * @since 1.0.0
     */
    name;
    /**
     * The comment parser of this signature.
     * @since 2.3.0
     */
    comment;
    /**
     * The type parameters of this signature.
     * @since 1.0.0
     */
    typeParameters;
    /**
     * The parameters of this signature.
     * @since 1.0.0
     */
    parameters;
    /**
     * The return type of this signature.
     * @since 1.0.0
     */
    returnType;
    constructor(data) {
        const { id, name, comment, typeParameters, parameters, returnType } = data;
        this.id = id;
        this.name = name;
        this.comment = comment;
        this.typeParameters = typeParameters;
        this.parameters = parameters;
        this.returnType = returnType;
    }
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this parser.
     */
    toJSON() {
        return {
            id: this.id,
            name: this.name,
            comment: this.comment.toJSON(),
            typeParameters: this.typeParameters.map((typeParameter) => typeParameter.toJSON()),
            parameters: this.parameters.map((parameter) => parameter.toJSON()),
            returnType: this.returnType.toJSON()
        };
    }
    /**
     * Generates a new {@link SignatureParser} instance from the given data.
     * @since 1.0.0
     * @param reflection The reflection to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromTypeDoc(reflection) {
        const { kind, id, name, comment = { summary: [] }, typeParameters = [], parameters = [], type } = reflection;
        if (![ReflectionKind.CallSignature, ReflectionKind.ConstructorSignature, ReflectionKind.IndexSignature].includes(kind)) {
            throw new Error(`Expected Signature (${ReflectionKind.CallSignature}, ${ReflectionKind.ConstructorSignature}, ${ReflectionKind.IndexSignature}), but received ${reflectionKindToString(kind)} (${kind}). NAME=${name};ID=${id}`);
        }
        return new SignatureParser({
            id,
            name,
            comment: CommentParser.generateFromTypeDoc(comment),
            typeParameters: typeParameters.map((typeParameter) => TypeParameterParser.generateFromTypeDoc(typeParameter)),
            parameters: parameters.map((parameter) => ParameterParser.generateFromTypeDoc(parameter)),
            returnType: TypeParser.generateFromTypeDoc(type)
        });
    }
    /**
     * Generates a new {@link ClassConstructorParser} instance from the given data.
     * @param json The json to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromJson(json) {
        const { id, name, comment, typeParameters, parameters, returnType } = json;
        return new SignatureParser({
            id,
            name,
            comment: CommentParser.generateFromJson(comment),
            typeParameters: typeParameters.map((typeParameter) => TypeParameterParser.generateFromJson(typeParameter)),
            parameters: parameters.map((parameter) => ParameterParser.generateFromJson(parameter)),
            returnType: TypeParser.generateFromJson(returnType)
        });
    }
}
//# sourceMappingURL=SignatureParser.js.map