import { TypeParser } from '#lib/structures/type-parsers';
import { ReflectionKind, reflectionKindToString } from '#lib/types';
/**
 * Parses data from a type parameter reflection.
 * @since 1.0.0
 */
export class TypeParameterParser {
    /**
     * The identifier of this parser.
     * @since 1.0.0
     */
    id;
    /**
     * The name of this type parameter.
     * @since 1.0.0
     */
    name;
    /**
     * The type of this type parameter.
     * @since 1.0.0
     */
    constraint;
    /**
     * The default value of this type parameter.
     * @since 1.0.0
     */
    default;
    constructor(data) {
        const { id, name, constraint, default: defaultValue } = data;
        this.id = id;
        this.name = name;
        this.constraint = constraint;
        this.default = defaultValue;
    }
    /**
     * Converts this type parameter to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this type parameter.
     */
    toJSON() {
        return {
            id: this.id,
            name: this.name,
            constraint: this.constraint ? this.constraint.toJSON() : null,
            default: this.default ? this.default.toJSON() : null
        };
    }
    /**
     * Generates a new {@link TypeParameterParser} instance from the given data.
     * @since 1.0.0
     * @param reflection The reflection to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromTypeDoc(reflection) {
        const { kind, id, name, type, default: _default } = reflection;
        if (kind !== ReflectionKind.TypeParameter) {
            throw new Error(`Expected TypeParameter (${ReflectionKind.TypeParameter}), but received ${reflectionKindToString(kind)} (${kind}). NAME=${name};ID=${id}`);
        }
        return new TypeParameterParser({
            id,
            name,
            constraint: type ? TypeParser.generateFromTypeDoc(type) : null,
            default: _default ? TypeParser.generateFromTypeDoc(_default) : null
        });
    }
    /**
     * Generates a new {@link ClassConstructorParser} instance from the given data.
     * @param json The json to generate the parser from.
     * @returns The generated parser.
     */
    static generateFromJson(json) {
        const { id, name, constraint, default: _default } = json;
        return new TypeParameterParser({
            id,
            name,
            constraint: constraint ? TypeParser.generateFromJson(constraint) : null,
            default: _default ? TypeParser.generateFromJson(_default) : null
        });
    }
}
//# sourceMappingURL=TypeParameterParser.js.map