import { TypeParser } from '#lib/structures/type-parsers';
/**
 * Parses data for a conditional type.
 * @since 1.0.0
 */
export class ConditionalTypeParser {
    /**
     * The kind of type this parser is for.
     * @since 1.0.0
     */
    kind = TypeParser.Kind.Conditional;
    /**
     * The check type of this conditional type.
     * @since 1.0.0
     */
    checkType;
    /**
     * The extends type of this conditional type.
     * @since 1.0.0
     */
    extendsType;
    /**
     * The type of this conditional type when the check type is true.
     * @since 1.0.0
     */
    trueType;
    /**
     * The type of this conditional type when the check type is false.
     * @since 1.0.0
     */
    falseType;
    constructor(data) {
        const { checkType, extendsType, trueType, falseType } = data;
        this.checkType = checkType;
        this.extendsType = extendsType;
        this.trueType = trueType;
        this.falseType = falseType;
    }
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this parser.
     */
    toJSON() {
        return {
            kind: this.kind,
            checkType: this.checkType.toJSON(),
            extendsType: this.extendsType.toJSON(),
            trueType: this.trueType.toJSON(),
            falseType: this.falseType.toJSON()
        };
    }
    /**
     * Converts this parser to a string.
     * @since 1.0.0
     * @returns The string representation of this parser.
     */
    toString(project) {
        return ConditionalTypeParser.formatToString({ parser: this, project });
    }
    /**
     * Formats this type parser to a string.
     * @since 4.0.0
     * @param options The options to format this type parser to a string.
     * @returns The string representation of this parser.
     */
    static formatToString(options) {
        const { parser, project } = options;
        return `${TypeParser.wrap(parser.checkType, TypeParser.BindingPowers[TypeParser.Kind.Conditional], project)} extends ${parser.extendsType.toString(project)} ? ${parser.trueType.toString(project)} : ${parser.falseType.toString(project)}`;
    }
}
//# sourceMappingURL=ConditionalTypeParser.js.map