import { TypeParser } from '#lib/structures/type-parsers';
/**
 * Parses data for a mapped type.
 * @since 1.0.0
 */
export class MappedTypeParser {
    /**
     * The kind of type this parser is for.
     * @since 1.0.0
     */
    kind = TypeParser.Kind.Mapped;
    /**
     * The parameter name of this mapped type.
     * @since 1.0.0
     */
    parameter;
    /**
     * The parameter type of this mapped type.
     * @since 1.0.0
     */
    parameterType;
    /**
     * The name type of this mapped type.
     * @since 1.0.0
     */
    nameType;
    /**
     * The template type of this mapped type.
     * @since 1.0.0
     */
    templateType;
    /**
     * The readonly modifier of this mapped type.
     * @since 1.0.0
     */
    readonly;
    /**
     * The optional modifier of this mapped type.
     * @since 1.0.0
     */
    optional;
    constructor(data) {
        const { parameter, parameterType, nameType, templateType, readonly, optional } = data;
        this.parameter = parameter;
        this.parameterType = parameterType;
        this.nameType = nameType;
        this.templateType = templateType;
        this.readonly = readonly;
        this.optional = optional;
    }
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this parser.
     */
    toJSON() {
        return {
            kind: this.kind,
            parameter: this.parameter,
            parameterType: this.parameterType.toJSON(),
            nameType: this.nameType ? this.nameType.toJSON() : null,
            templateType: this.templateType.toJSON(),
            readonly: this.readonly,
            optional: this.optional
        };
    }
    /**
     * Converts this parser to a string.
     * @since 1.0.0
     * @returns The string representation of this parser.
     */
    toString(project) {
        return MappedTypeParser.formatToString({ parser: this, project });
    }
    /**
     * Formats this type parser to a string.
     * @since 4.0.0
     * @param options The options to format this type parser to a string.
     * @returns The string representation of this parser.
     */
    static formatToString(options) {
        const { parser, project } = options;
        const readonly = parser.readonly === MappedTypeParser.Modifier.Add ? 'readonly' : parser.readonly === MappedTypeParser.Modifier.Remove ? '-readonly' : '';
        const optional = parser.optional === MappedTypeParser.Modifier.Add ? '?' : parser.optional === MappedTypeParser.Modifier.Remove ? '-?' : '';
        return `{ ${readonly}[${parser.parameter} in ${parser.parameterType.toString(project)}]${optional}: ${parser.templateType.toString(project)} }`;
    }
}
(function (MappedTypeParser) {
    /**
     * The modifier for a mapped type.
     */
    let Modifier;
    (function (Modifier) {
        Modifier["Add"] = "+";
        Modifier["Remove"] = "-";
    })(Modifier = MappedTypeParser.Modifier || (MappedTypeParser.Modifier = {}));
})(MappedTypeParser || (MappedTypeParser = {}));
//# sourceMappingURL=MappedTypeParser.js.map