import { TypeParser } from '#lib/structures/type-parsers';
/**
 * Parses data for a reference type.
 * @since 1.0.0
 */
export class ReferenceTypeParser {
    /**
     * The kind of type this parser is for.
     * @since 1.0.0
     */
    kind = TypeParser.Kind.Reference;
    /**
     * The id of this reference type.
     * @since 1.0.0
     */
    id;
    /**
     * The name of this reference type.
     * @since 1.0.0
     */
    name;
    /**
     * The package name of this reference type.
     * @since 1.0.0
     */
    packageName;
    /**
     * The package version of this reference type.
     * @since 10.0.0
     */
    packageVersion;
    /**
     * The type arguments of this reference type.
     * @since 1.0.0
     */
    typeArguments;
    constructor(data) {
        const { id, name, packageName, packageVersion, typeArguments } = data;
        this.id = id;
        this.name = name;
        this.packageName = packageName;
        this.packageVersion = packageVersion;
        this.typeArguments = typeArguments;
    }
    /**
     * Whether this reference type is from a package.
     * @since 1.0.0
     * @returns
     */
    isPackage() {
        return this.packageName !== null;
    }
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this parser.
     */
    toJSON() {
        return {
            kind: this.kind,
            id: this.id,
            name: this.name,
            packageName: this.packageName,
            packageVersion: this.packageVersion,
            typeArguments: this.typeArguments.map((type) => type.toJSON())
        };
    }
    /**
     * Converts this parser to a string.
     * @since 1.0.0
     * @param project The project to convert this parser to a string.
     * @returns The string representation of this parser.
     */
    toString(project) {
        return ReferenceTypeParser.formatToString({ parser: this, project });
    }
    /**
     * Formats this type parser to a string.
     * @since 4.0.0
     * @param options The options to format this type parser to a string.
     * @returns The string representation of this parser.
     */
    static formatToString(options) {
        const { parser, project } = options;
        const typeArguments = parser.typeArguments.length > 0 ? `<${parser.typeArguments.map((type) => type.toString(project)).join(', ')}>` : '';
        return `${parser.packageName ? `${parser.packageName}.` : ''}${parser.name}${typeArguments}`;
    }
}
//# sourceMappingURL=ReferenceTypeParser.js.map