import { MethodParser, PropertyParser, SignatureParser } from '#lib/structures/misc';
import { TypeParser } from '#lib/structures/type-parsers';
/**
 * Parses data for a reflection type.
 * @since 1.0.0
 */
export class ReflectionTypeParser {
    /**
     * The kind of type this parser is for.
     * @since 1.0.0
     */
    kind = TypeParser.Kind.Reflection;
    /**
     * The properties of this reflection type.
     * @since 8.0.0
     */
    properties;
    /**
     * The signatures of this reflection type.
     * @since 8.0.0
     */
    signatures;
    /**
     * The methods of this reflection type.
     * @since 8.1.0
     */
    methods;
    constructor(data) {
        const { properties, signatures, methods } = data;
        this.properties = properties;
        this.signatures = signatures;
        this.methods = methods;
    }
    /**
     * Converts this parser to a json compatible format.
     * @since 1.0.0
     * @returns The json compatible format of this parser.
     */
    toJSON() {
        return {
            kind: this.kind,
            properties: this.properties?.map((property) => property.toJSON()) ?? null,
            signatures: this.signatures?.map((signature) => signature.toJSON()) ?? null,
            methods: this.methods?.map((method) => method.toJSON()) ?? null
        };
    }
    /**
     * Converts this parser to a string.
     * @since 1.0.0
     * @returns The string representation of this parser.
     */
    toString(project) {
        return ReflectionTypeParser.formatToString({ parser: this, project });
    }
    /**
     * Formats this type parser to a string.
     * @since 4.0.0
     * @param options The options to format this type parser to a string.
     * @returns The string representation of this parser.
     */
    static formatToString(options) {
        const { parser, project } = options;
        return `{ ${parser.properties?.length ? `${parser.properties.map((property) => `${property.name}: ${property.type.toString(project)};`).join(' ')} ` : ''}${parser.signatures?.length
            ? `${parser.signatures
                .map((signature) => `${signature.typeParameters.length
                ? `<${signature.typeParameters
                    .map((typeParameter) => `${typeParameter.name}${typeParameter.constraint ? ` extends ${typeParameter.constraint.toString(project)}` : ''}${typeParameter.default ? ` = ${typeParameter.default.toString(project)}` : ''}`)
                    .join(', ')}>`
                : ''}(${signature.parameters.length
                ? signature.parameters
                    .map((parameter) => `${parameter.rest ? '...' : ''}${parameter.name}${parameter.optional ? '?:' : ':'} ${parameter.type.toString(project)}`)
                    .join(', ')
                : ''}): ${signature.returnType.toString(project)};`)
                .join(' ')} `
            : ''}${parser.methods?.length
            ? `${parser.methods.map((method) => method.signatures.length
                ? method.signatures.length
                    ? method.signatures
                        .map((signature) => `${signature.typeParameters.length
                        ? `<${signature.typeParameters
                            .map((typeParameter) => `${typeParameter.name}${typeParameter.constraint ? ` extends ${typeParameter.constraint.toString(project)}` : ''}${typeParameter.default ? ` = ${typeParameter.default.toString(project)}` : ''}`)
                            .join(', ')}>`
                        : ''}(${signature.parameters.length
                        ? signature.parameters
                            .map((parameter) => `${parameter.rest ? '...' : ''}${parameter.name}${parameter.optional ? '?:' : ':'} ${parameter.type.toString(project)}`)
                            .join(', ')
                        : ''}): ${signature.returnType.toString(project)};`)
                        .join(' ')
                    : ''
                : '')}`
            : ''}}`;
    }
}
//# sourceMappingURL=ReflectionTypeParser.js.map