import { ProjectParser } from '#lib/structures/ProjectParser';
import { MethodParser, PropertyParser, SignatureParser } from '#lib/structures/misc';
import { ArrayTypeParser, ConditionalTypeParser, IndexedAccessTypeParser, InferredTypeParser, IntersectionTypeParser, IntrinsicTypeParser, LiteralTypeParser, MappedTypeParser, NamedTupleMemberTypeParser, OptionalTypeParser, PredicateTypeParser, QueryTypeParser, ReferenceTypeParser, ReflectionTypeParser, RestTypeParser, TemplateLiteralTypeParser, TupleTypeParser, TypeOperatorTypeParser, UnionTypeParser, UnknownTypeParser } from '#lib/structures/type-parsers';
import { ReflectionKind } from '#lib/types';
import {} from 'typedoc';
export var TypeParser;
(function (TypeParser) {
    TypeParser.dependencies = new Map();
    /**
     * Generates a new {@link TypeParser} instance from the given data.
     * @since 1.0.0
     * @param type The type to generate the parser from.
     * @returns The generated parser.
     */
    function generateFromTypeDoc(type) {
        switch (type.type) {
            case 'array': {
                const { elementType } = type;
                return new ArrayTypeParser({ type: generateFromTypeDoc(elementType) });
            }
            case 'conditional': {
                const { checkType, extendsType, trueType, falseType } = type;
                return new ConditionalTypeParser({
                    checkType: generateFromTypeDoc(checkType),
                    extendsType: generateFromTypeDoc(extendsType),
                    trueType: generateFromTypeDoc(trueType),
                    falseType: generateFromTypeDoc(falseType)
                });
            }
            case 'indexedAccess': {
                const { objectType, indexType } = type;
                return new IndexedAccessTypeParser({
                    objectType: generateFromTypeDoc(objectType),
                    indexType: generateFromTypeDoc(indexType)
                });
            }
            case 'inferred': {
                const { name } = type;
                return new InferredTypeParser({ type: name });
            }
            case 'intersection': {
                const { types } = type;
                return new IntersectionTypeParser({ types: types.map((type) => generateFromTypeDoc(type)) });
            }
            case 'intrinsic': {
                const { name } = type;
                return new IntrinsicTypeParser({ type: name });
            }
            case 'literal': {
                const { value } = type;
                return new LiteralTypeParser({ value: (typeof value === 'object' && value !== null ? value.value : value)?.toString() ?? 'null' });
            }
            case 'mapped': {
                const { parameter, parameterType, nameType, templateType, optionalModifier, readonlyModifier } = type;
                return new MappedTypeParser({
                    parameter,
                    parameterType: generateFromTypeDoc(parameterType),
                    nameType: nameType ? generateFromTypeDoc(nameType) : null,
                    templateType: generateFromTypeDoc(templateType),
                    optional: (optionalModifier ?? null),
                    readonly: (readonlyModifier ?? null)
                });
            }
            case 'namedTupleMember': {
                const { element, isOptional, name } = type;
                return new NamedTupleMemberTypeParser({
                    name,
                    type: generateFromTypeDoc(element),
                    optional: isOptional ?? false
                });
            }
            case 'optional': {
                const { elementType } = type;
                return new OptionalTypeParser({ type: generateFromTypeDoc(elementType) });
            }
            case 'predicate': {
                const { asserts, name, targetType } = type;
                return new PredicateTypeParser({
                    asserts,
                    name,
                    type: targetType ? generateFromTypeDoc(targetType) : null
                });
            }
            case 'query': {
                const { queryType } = type;
                return new QueryTypeParser({ query: generateFromTypeDoc(queryType) });
            }
            case 'reference': {
                const { target, name, package: _package, qualifiedName, typeArguments = [] } = type;
                const packageVersion = TypeParser.dependencies.get(name);
                return new ReferenceTypeParser({
                    id: typeof target === 'number' ? target : null,
                    name: qualifiedName ?? name,
                    packageName: _package ?? null,
                    packageVersion: packageVersion ?? null,
                    typeArguments: typeArguments.map((type) => generateFromTypeDoc(type))
                });
            }
            case 'reflection': {
                const { declaration } = type;
                const properties = declaration.children?.filter((child) => child.kind === ReflectionKind.Property).map((child) => PropertyParser.generateFromTypeDoc(child)) ??
                    null;
                const methods = declaration.children?.filter((child) => child.kind === ReflectionKind.Method).map((child) => MethodParser.generateFromTypeDoc(child)) ??
                    null;
                return new ReflectionTypeParser({
                    properties: properties?.length ? properties : null,
                    signatures: declaration.signatures?.map((signature) => SignatureParser.generateFromTypeDoc(signature)) ?? null,
                    methods: methods?.length ? methods : null
                });
            }
            case 'rest': {
                const { elementType } = type;
                return new RestTypeParser({ type: generateFromTypeDoc(elementType) });
            }
            case 'templateLiteral': {
                const { head, tail } = type;
                return new TemplateLiteralTypeParser({
                    head,
                    tail: tail.map(([type, text]) => ({ type: generateFromTypeDoc(type), text }))
                });
            }
            case 'tuple': {
                const { elements = [] } = type;
                return new TupleTypeParser({ types: elements.map((type) => generateFromTypeDoc(type)) });
            }
            case 'typeOperator': {
                const { operator, target } = type;
                return new TypeOperatorTypeParser({
                    operator: operator,
                    type: generateFromTypeDoc(target)
                });
            }
            case 'union': {
                const { types } = type;
                return new UnionTypeParser({ types: types.map((type) => generateFromTypeDoc(type)) });
            }
            case 'unknown': {
                const { name } = type;
                return new UnknownTypeParser({ name });
            }
        }
    }
    TypeParser.generateFromTypeDoc = generateFromTypeDoc;
    /**
     * Generates a new {@link TypeParser} instance from the given data.
     * @param json The json to generate the parser from.
     * @returns The generated parser.
     */
    function generateFromJson(json) {
        switch (json.kind) {
            case Kind.Array: {
                const { type } = json;
                return new ArrayTypeParser({ type: generateFromJson(type) });
            }
            case Kind.Conditional: {
                const { checkType, extendsType, trueType, falseType } = json;
                return new ConditionalTypeParser({
                    checkType: generateFromJson(checkType),
                    extendsType: generateFromJson(extendsType),
                    trueType: generateFromJson(trueType),
                    falseType: generateFromJson(falseType)
                });
            }
            case Kind.IndexedAccess: {
                const { objectType, indexType } = json;
                return new IndexedAccessTypeParser({
                    objectType: generateFromJson(objectType),
                    indexType: generateFromJson(indexType)
                });
            }
            case Kind.Inferred: {
                const { type } = json;
                return new InferredTypeParser({ type });
            }
            case Kind.Intersection: {
                const { types } = json;
                return new IntersectionTypeParser({ types: types.map((type) => generateFromJson(type)) });
            }
            case Kind.Intrinsic: {
                const { type } = json;
                return new IntrinsicTypeParser({ type });
            }
            case Kind.Literal: {
                const { value } = json;
                return new LiteralTypeParser({ value });
            }
            case Kind.Mapped: {
                const { parameter, parameterType, nameType, templateType, optional, readonly } = json;
                return new MappedTypeParser({
                    parameter,
                    parameterType: generateFromJson(parameterType),
                    nameType: nameType ? generateFromJson(nameType) : null,
                    templateType: generateFromJson(templateType),
                    optional,
                    readonly
                });
            }
            case Kind.NamedTupleMember: {
                const { type, optional, name } = json;
                return new NamedTupleMemberTypeParser({ name, type: generateFromJson(type), optional });
            }
            case Kind.Optional: {
                const { type } = json;
                return new OptionalTypeParser({ type: generateFromJson(type) });
            }
            case Kind.Predicate: {
                const { asserts, name, type } = json;
                return new PredicateTypeParser({ asserts, name, type: type ? generateFromJson(type) : null });
            }
            case Kind.Query: {
                const { query } = json;
                return new QueryTypeParser({ query: generateFromJson(query) });
            }
            case Kind.Reference: {
                const { id, name, packageName, packageVersion, typeArguments } = json;
                return new ReferenceTypeParser({
                    id,
                    name,
                    packageName,
                    packageVersion,
                    typeArguments: typeArguments.map((type) => generateFromJson(type))
                });
            }
            case Kind.Reflection: {
                const { properties, signatures, methods } = json;
                return new ReflectionTypeParser({
                    properties: properties?.map((property) => PropertyParser.generateFromJson(property)) ?? null,
                    signatures: signatures?.map((signature) => SignatureParser.generateFromJson(signature)) ?? null,
                    methods: methods?.map((method) => MethodParser.generateFromJson(method)) ?? null
                });
            }
            case Kind.Rest: {
                const { type } = json;
                return new RestTypeParser({ type: generateFromJson(type) });
            }
            case Kind.TemplateLiteral: {
                const { head, tail } = json;
                return new TemplateLiteralTypeParser({
                    head,
                    tail: tail.map((tail) => ({ type: generateFromJson(tail.type), text: tail.text }))
                });
            }
            case Kind.Tuple: {
                const { types } = json;
                return new TupleTypeParser({ types: types.map((type) => generateFromJson(type)) });
            }
            case Kind.TypeOperator: {
                const { operator, type } = json;
                return new TypeOperatorTypeParser({ operator, type: generateFromJson(type) });
            }
            case Kind.Union: {
                const { types } = json;
                return new UnionTypeParser({ types: types.map((type) => generateFromJson(type)) });
            }
            case Kind.Unknown: {
                const { name } = json;
                return new UnknownTypeParser({ name });
            }
        }
    }
    TypeParser.generateFromJson = generateFromJson;
    /**
     * Wraps the given type parser depending on it's binding power.
     * @since 1.0.0
     * @param type The type parser to wrap.
     * @param binding The binding power of the type parser.
     * @returns The wrapped type parser.
     */
    function wrap(type, binding, project) {
        return TypeParser.BindingPowers[type.kind] < binding ? `(${type.toString(project)})` : type.toString(project);
    }
    TypeParser.wrap = wrap;
    /**
     * The kind of type parser.
     * @since 1.0.0
     */
    let Kind;
    (function (Kind) {
        Kind["Array"] = "array";
        Kind["Conditional"] = "conditional";
        Kind["IndexedAccess"] = "indexedAccess";
        Kind["Inferred"] = "inferred";
        Kind["Intersection"] = "intersection";
        Kind["Intrinsic"] = "intrinsic";
        Kind["Literal"] = "literal";
        Kind["Mapped"] = "mapped";
        Kind["NamedTupleMember"] = "namedTupleMember";
        Kind["Optional"] = "optional";
        Kind["Predicate"] = "predicate";
        Kind["Query"] = "query";
        Kind["Reference"] = "reference";
        Kind["Reflection"] = "reflection";
        Kind["Rest"] = "rest";
        Kind["TemplateLiteral"] = "templateLiteral";
        Kind["Tuple"] = "tuple";
        Kind["TypeOperator"] = "typeOperator";
        Kind["Union"] = "union";
        Kind["Unknown"] = "unknown";
    })(Kind = TypeParser.Kind || (TypeParser.Kind = {}));
    /**
     * The binding powers of the type parsers.
     * @since 1.0.0
     */
    TypeParser.BindingPowers = {
        [Kind.Array]: 999,
        [Kind.Conditional]: 150,
        [Kind.IndexedAccess]: 999,
        [Kind.Inferred]: 999,
        [Kind.Intersection]: 120,
        [Kind.Intrinsic]: 999,
        [Kind.Literal]: 999,
        [Kind.Mapped]: 999,
        [Kind.NamedTupleMember]: 999,
        [Kind.Optional]: 999,
        [Kind.Predicate]: 999,
        [Kind.Query]: 900,
        [Kind.Reference]: 999,
        [Kind.Reflection]: 999,
        [Kind.Rest]: 999,
        [Kind.TemplateLiteral]: 999,
        [Kind.Tuple]: 999,
        [Kind.TypeOperator]: 900,
        [Kind.Union]: 100,
        [Kind.Unknown]: -1
    };
})(TypeParser || (TypeParser = {}));
//# sourceMappingURL=TypeParser.js.map